import os

import numpy as np
import numpy.testing as npt
from scipy.signal import signaltools

import nitime
from nitime import algorithms as tsa
from nitime import utils as ut

def test_scipy_resample():
    """ Tests scipy signal's resample function
    """
    # create a freq list with max freq < 16 Hz
    freq_list = np.random.randint(0,high=15,size=5)
    # make a test signal with sampling freq = 64 Hz
    a = [np.sin(2*np.pi*f*np.linspace(0,1,64,endpoint=False))
         for f in freq_list]
    tst = np.array(a).sum(axis=0)
    # interpolate to 128 Hz sampling
    t_up = signaltools.resample(tst, 128)
    np.testing.assert_array_almost_equal(t_up[::2], tst)
    # downsample to 32 Hz
    t_dn = signaltools.resample(tst, 32)
    np.testing.assert_array_almost_equal(t_dn, tst[::2])

    # downsample to 48 Hz, and compute the sampling analytically for comparison
    dn_samp_ana = np.array([np.sin(2*np.pi*f*np.linspace(0,1,48,endpoint=False))
                            for f in freq_list]).sum(axis=0)
    t_dn2 = signaltools.resample(tst, 48)
    npt.assert_array_almost_equal(t_dn2, dn_samp_ana)

def test_coherency_mlab():
    """Tests that the coherency algorithm runs smoothly, using the mlab csd
    routine, that the resulting matrix is symmetric and that the frequency bands
    in the output make sense"""
    
    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])

    method = {"this_method":'mlab',
              "NFFT":256,
              "Fs":2*np.pi}

    f,c = tsa.coherency(np.vstack([x,y]),csd_method=method)

    npt.assert_array_almost_equal(c[0,1],c[1,0].conjugate())
    npt.assert_array_almost_equal(c[0,0],np.ones(f.shape))
    f_theoretical = ut.get_freqs(method['Fs'],method['NFFT'])
    npt.assert_array_almost_equal(f,f_theoretical)

def test_coherency_multi_taper():
    """Tests that the coherency algorithm runs smoothly, using the multi_taper
    csd routine and that the resulting matrix is symmetric"""
    
    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])

    method = {"this_method":'multi_taper_csd',
              "Fs":2*np.pi}

    f,c = tsa.coherency(np.vstack([x,y]),csd_method=method)

    npt.assert_array_almost_equal(c[0,1],c[1,0].conjugate())
    npt.assert_array_almost_equal(c[0,0],np.ones(f.shape))

def test_coherence_mlab():
    """Tests that the code runs and that the resulting matrix is symmetric """  

    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])

    method = {"this_method":'mlab',
              "NFFT":256,
              "Fs":2*np.pi}
    
    f,c = tsa.coherence(np.vstack([x,y]),csd_method=method)
    np.testing.assert_array_almost_equal(c[0,1],c[1,0])

    f_theoretical = ut.get_freqs(method['Fs'],method['NFFT'])
    npt.assert_array_almost_equal(f,f_theoretical)

def test_coherence_multi_taper():
    """Tests that the code runs and that the resulting matrix is symmetric """  

    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])

    method = {"this_method":'multi_taper_csd',
              "Fs":2*np.pi}
     
    f,c = tsa.coherence(np.vstack([x,y]),csd_method=method)
    npt.assert_array_almost_equal(c[0,1],c[1,0])

def test_coherence_partial():
    """ Test partial coherence"""

    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])
    z = x + np.random.rand(t.shape[-1])

    method = {"this_method":'mlab',
              "NFFT":256,
              "Fs":2*np.pi}
    f,c = tsa.coherence_partial(np.vstack([x,y]),z,csd_method=method)

    f_theoretical = ut.get_freqs(method['Fs'],method['NFFT'])
    npt.assert_array_almost_equal(f,f_theoretical)
    npt.assert_array_almost_equal(c[0,1],c[1,0])

    
def test_coherency_cached():
    """Tests that the cached coherency gives the same result as the standard
    coherency"""

    t = np.linspace(0,16*np.pi,1024)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + np.random.rand(t.shape[-1])
    y = x + np.random.rand(t.shape[-1])

    f1,c1 = tsa.coherency(np.vstack([x,y]))

    ij = [(0,1),(1,0)]
    f2,cache = tsa.cache_fft(np.vstack([x,y]),ij)

    c2 = tsa.cache_to_coherency(cache,ij)

    npt.assert_array_almost_equal(c1[1,0],c2[1,0])
    npt.assert_array_almost_equal(c1[0,1],c2[0,1])


# XXX FIXME: http://github.com/nipy/nitime/issues/issue/1
@npt.dec.skipif(True) 
def test_coherence_linear_dependence():
    """
    Tests that the coherence between two linearly dependent time-series
    behaves as expected.
    
    From William Wei's book, according to eq. 14.5.34, if two time-series are
    linearly related through:

    y(t)  = alpha*x(t+time_shift)

    then the coherence between them should be equal to:

    .. :math:
    
    C(\nu) = \frac{1}{1+\frac{fft_{noise}(\nu)}{fft_{x}(\nu) \cdot \alpha^2}}
    
    """
    t = np.linspace(0,16*np.pi,2**14)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + 0.1 *np.random.rand(t.shape[-1])
    N = x.shape[-1]

    alpha = 10
    m = 3
    noise = 0.1 * np.random.randn(t.shape[-1])
    y = alpha*(np.roll(x,m)) + noise

    f_noise = np.fft.fft(noise)[0:N/2]
    f_x = np.fft.fft(x)[0:N/2]

    c_t = ( 1/( 1 + ( f_noise/( f_x*(alpha**2)) ) ) )

    method = {"this_method":'mlab',
              "NFFT":2048,
              "Fs":2*np.pi}

    f,c = tsa.coherence(np.vstack([x,y]),csd_method=method)
    c_t = np.abs(signaltools.resample(c_t,c.shape[-1]))

    npt.assert_array_almost_equal(c[0,1],c_t,2)
    
@npt.dec.skipif(True)
def test_coherence_phase_spectrum ():
    assert False, "Test Not Implemented"

@npt.dec.skipif(True)
def test_coherency_bavg():
    assert False, "Test Not Implemented"

@npt.dec.skipif(True)
def test_coherence_partial():
    assert False, "Test Not Implemented"

@npt.dec.skipif(True)
def test_coherence_partial_bavg():
    assert False, "Test Not Implemented"

#XXX def test_coherency_phase ()
#XXX def test_coherence_partial_phase()

@npt.dec.skipif(True)
def test_fir():
    assert False, "Test Not Implemented"

@npt.dec.skipif(True)
def test_percent_change():
    assert False, "Test Not Implemented"

def test_DPSS_windows():
    "Are the eigenvalues representing spectral concentration near unity"
    # these values from Percival and Walden 1993
    _, l = tsa.DPSS_windows(31, 6, 4)
    unos = np.ones(4)
    yield npt.assert_array_almost_equal, l, unos 
    _, l = tsa.DPSS_windows(31, 7, 4)
    yield npt.assert_array_almost_equal, l, unos 
    _, l = tsa.DPSS_windows(31, 8, 4)
    yield npt.assert_array_almost_equal, l, unos
                
def test_yule_walker_AR():
    arsig,_,_ = ut.ar_generator(N=512)
    avg_pwr = (arsig*arsig.conjugate()).mean()
    w, psd = tsa.yule_AR_est(arsig, 8, 1024)
    # for efficiency, let's leave out the 2PI in the numerator and denominator
    # for the following integral
    dw = 1./1024
    avg_pwr_est = np.trapz(psd, dx=dw)
    npt.assert_almost_equal(avg_pwr, avg_pwr_est, decimal=0)

def test_LD_AR():
    arsig,_,_ = ut.ar_generator(N=512)
    avg_pwr = (arsig*arsig.conjugate()).mean()
    w, psd = tsa.LD_AR_est(arsig, 8, 1024)
    # for efficiency, let's leave out the 2PI in the numerator and denominator
    # for the following integral
    dw = 1./1024
    avg_pwr_est = np.trapz(psd, dx=dw)
    npt.assert_almost_equal(avg_pwr, avg_pwr_est, decimal=0)
    
def test_periodogram():
    arsig,_,_ = ut.ar_generator(N=512)
    avg_pwr = (arsig*arsig.conjugate()).mean()
    f, psd = tsa.periodogram(arsig, N=2048)
    # for efficiency, let's leave out the 2PI in the numerator and denominator
    # for the following integral
    dw = 1./2048
    avg_pwr_est = np.trapz(psd, dx=dw)
    npt.assert_almost_equal(avg_pwr, avg_pwr_est, decimal=1)
    
def permutation_system(N):
    p = np.zeros((N,N))
    targets = range(N)
    for i in xrange(N):
        popper = np.random.randint(0, high=len(targets))
        j = targets.pop(popper)
        p[i,j] = 1
    return p

def test_boxcar_filter():
    a = np.random.rand(100)
    b = tsa.boxcar_filter(a)
    npt.assert_equal(a,b)

    #Should also work for odd number of elements:
    a = np.random.rand(99)
    b = tsa.boxcar_filter(a)
    npt.assert_equal(a,b)

    b = tsa.boxcar_filter(a,ub=0.25)
    npt.assert_equal(a.shape,b.shape)

    b = tsa.boxcar_filter(a,lb=0.25)
    npt.assert_equal(a.shape,b.shape)

def test_get_spectra():
    """Testing get_spectra"""
    t = np.linspace(0,16*np.pi,2**14)
    x = np.sin(t) + np.sin(2*t) + np.sin(3*t) + 0.1 *np.random.rand(t.shape[-1])
    x = np.reshape(x,(2,x.shape[-1]/2))
    N = x.shape[-1]

    #Make sure you get back the expected shape for different spectra: 
    NFFT = 64
    f_mlab=tsa.get_spectra(x,method={'this_method':'mlab','NFFT':NFFT})
    f_periodogram=tsa.get_spectra(x,method={'this_method':'periodogram_csd'})
    f_multi_taper=tsa.get_spectra(x,method={'this_method':'multi_taper_csd'})

    npt.assert_equal(f_mlab[0].shape[0],NFFT/2+1)
    npt.assert_equal(f_periodogram[0].shape[0],N/2+1)
    npt.assert_equal(f_multi_taper[0].shape[0],N/2+1)

def test_psd_matlab():

    """ Test the results of mlab csd/psd against saved results from Matlab"""

    from matplotlib import mlab

    test_dir_path = os.path.join(nitime.__path__[0],'tests')
    
    ts = np.loadtxt(os.path.join(test_dir_path,'tseries12.txt'))
    
    #Complex signal! 
    ts0 = ts[1] + ts[0]*np.complex(0,1) 

    NFFT = 256;
    Fs = 1.0;
    noverlap = NFFT/2

    fxx, f = mlab.psd(ts0,NFFT=NFFT,Fs=Fs,noverlap=noverlap,
                      scale_by_freq=True)

    fxx_mlab = np.fft.fftshift(fxx).squeeze()

    fxx_matlab = np.loadtxt(os.path.join(test_dir_path,'fxx_matlab.txt'))

    npt.assert_almost_equal(fxx_mlab,fxx_matlab,decimal=5)

def test_coherence_matlab():

    """ Test against coherence values calculated with matlab's mscohere"""
    test_dir_path = os.path.join(nitime.__path__[0],'tests')

    ts = np.loadtxt(os.path.join(test_dir_path,'tseries12.txt'))

    ts0 = ts[1]   
    ts1 = ts[0]  

    method = {}
    method['this_method']='mlab'
    method['NFFT'] = 64;
    method['Fs'] = 1.0;
    method['noverlap'] = method['NFFT']/2

    ttt = np.vstack([ts0,ts1])
    f,cxy_mlab = tsa.coherence(ttt,csd_method=method)
    cxy_matlab = np.loadtxt(os.path.join(test_dir_path,'cxy_matlab.txt'))

    npt.assert_almost_equal(cxy_mlab[0][1],cxy_matlab,decimal=5)



import numpy as np
import numpy.testing as npt
from nitime import utils as ut
import nitime.timeseries as ts
import nitime.analysis as nta
import nose.tools as nt
import decotest

def test_SpectralAnalyzer():

    Fs = np.pi
    t = np.arange(1024)
    x = np.sin(10*t) + np.random.rand(t.shape[-1])
    y = np.sin(10*t) + np.random.rand(t.shape[-1])

    T = ts.TimeSeries(np.vstack([x,y]),sampling_rate=Fs)

    C = nta.SpectralAnalyzer(T)

    f,c = C()

    npt.assert_equal(f.shape,(33,)) #This is the setting for this analyzer
                                    #(window-length of 64)
    npt.assert_equal(c.shape,(2,2,33))

    f,c = C.spectrum_fourier

    npt.assert_equal(f.shape,(t.shape[0]/2+1,))
    npt.assert_equal(c.shape,(2,t.shape[0]/2+1))

    f,c = C.spectrum_multi_taper

    npt.assert_equal(f.shape,(t.shape[0]/2+1,))
    npt.assert_equal(c.shape,(2,2,t.shape[0]/2+1))

def test_CoherenceAnalyzer():
    Fs = np.pi
    t = np.arange(10)
    x = np.sin(10*t) + np.random.rand(t.shape[-1])
    y = np.sin(10*t) + np.random.rand(t.shape[-1])
    T = ts.TimeSeries(np.vstack([x,y]),sampling_rate=Fs)
    C = nta.CoherenceAnalyzer(T)

    npt.assert_equal(C().shape,(2,2,33)) #Default mlab_method
    #Coherence symmetry:
    npt.assert_equal(C.coherence[0,1],C.coherence[1,0])
    #Phase/delay asymmetry:
    npt.assert_equal(C.phase[0,1],-1*C.phase[1,0])
    npt.assert_equal(C.delay[0,1][1:],-1*C.delay[1,0][1:]) #The very first one
                                        #is a nan

    #Calculation of the spectrum is the same as in the default spectral analyzer:
    S = nta.SpectralAnalyzer(T)
    npt.assert_equal(S(),(C.frequencies,C.spectrum))

def test_SparseCoherenceAnalyzer():
    Fs = np.pi
    t = np.arange(10)
    x = np.sin(10*t) + np.random.rand(t.shape[-1])
    y = np.sin(10*t) + np.random.rand(t.shape[-1])
    T = ts.TimeSeries(np.vstack([x,y]),sampling_rate=Fs)
    C = nta.SparseCoherenceAnalyzer(T,ij=((0,1),(1,0)))

    #Coherence symmetry:
    npt.assert_equal(np.abs(C[0,1]),np.abs(C[1,0]))

def test_CorrelationAnalyzer():

    Fs = np.pi
    t = np.arange(1024)
    x = np.sin(10*t) + np.random.rand(t.shape[-1])
    y = np.sin(10*t) + np.random.rand(t.shape[-1])

    T = ts.TimeSeries(np.vstack([x,y]),sampling_rate=Fs)

    C = nta.CorrelationAnalyzer(T)

    #Test the symmetry: correlation(x,y)==correlation(y,x) 
    npt.assert_equal(C[0,1],C[1,0])
    #Test the self-sameness: correlation(x,x)==1
    npt.assert_equal(C[0,0],1)
    npt.assert_equal(C[1,1],1)

    #Test the cross-correlation:
    #First the symmetry:
    npt.assert_array_almost_equal(C.xcorr.data[0,1],C.xcorr.data[1,0])
    
    #Test the normalized cross-correlation
    #The cross-correlation should be equal to the correlation at time-lag 0
    npt.assert_equal(C.xcorr_norm.data[0,1,C.xcorr_norm.time==0]
                            ,C[0,1])

    #And the auto-correlation should be equal to 1 at 0 time-lag:
    npt.assert_equal(C.xcorr_norm.data[0,0,C.xcorr_norm.time==0],1)

    #Does it depend on having an even number of time-points?
    #make another time-series with an odd number of items:
    t = np.arange(1023)
    x = np.sin(10*t) + np.random.rand(t.shape[-1])
    y = np.sin(10*t) + np.random.rand(t.shape[-1])

    T = ts.TimeSeries(np.vstack([x,y]),sampling_rate=Fs)

    C = nta.CorrelationAnalyzer(T)
    
    npt.assert_equal(C.xcorr_norm.data[0,1,C.xcorr_norm.time==0]
                            ,C[0,1])

def test_EventRelatedAnalyzer():

    cycles = 10
    l = 1024
    unit = 2*np.pi/l
    t = np.arange(0,2*np.pi+unit,unit)
    signal = np.sin(cycles*t)
    events = np.zeros(t.shape)
    #Zero crossings: 
    idx = np.where(np.abs(signal)<0.03)[0]
    #An event occurs at the beginning of every cycle:
    events[idx[:-2:2]]=1
    #and another kind of event at the end of each cycle:
    events[idx[1:-1:2]]=2

    T_signal = ts.TimeSeries(signal,sampling_rate=1)
    T_events = ts.TimeSeries(events,sampling_rate=1)
    ETA = nta.EventRelatedAnalyzer(T_signal,T_events,l/(cycles*2)).eta

    #This looks good, but doesn't pass unless you consider 3 digits:
    npt.assert_almost_equal(ETA.data[0],signal[:ETA.data.shape[-1]],3)
    npt.assert_almost_equal(ETA.data[1],-1*signal[:ETA.data.shape[-1]],3)

    #Same should be true for the FIR analysis: 
    FIR = nta.EventRelatedAnalyzer(T_signal,T_events,l/(cycles*2)).FIR
    npt.assert_almost_equal(FIR.data[0],signal[:FIR.data.shape[-1]],3)
    npt.assert_almost_equal(FIR.data[1],-1*signal[:FIR.data.shape[-1]],3)

    #More dimensions: 
    T_signal = ts.TimeSeries(np.vstack([signal,signal]),sampling_rate=1)
    T_events = ts.TimeSeries(np.vstack([events,events]),sampling_rate=1)
    ETA = nta.EventRelatedAnalyzer(T_signal,T_events,l/(cycles*2)).eta

    #The events input and the time-series input have different dimensions:
    T_events = ts.TimeSeries(events,sampling_rate=1)
    ETA = nta.EventRelatedAnalyzer(T_signal,T_events,l/(cycles*2)).eta
    npt.assert_almost_equal(ETA.data[0][0],signal[:ETA.data.shape[-1]],3)
    npt.assert_almost_equal(ETA.data[1][1],-1*signal[:ETA.data.shape[-1]],3)
    
    #Input is an Events object, instead of a time-series:
    ts1 = ts.TimeSeries(np.arange(100),sampling_rate=1)
    ev = ts.Events([10,20,30])
    et = nta.EventRelatedAnalyzer(ts1,ev,5)

    #The five points comprising the average of the three sequences:
    npt.assert_equal(et.eta.data,[20.,21.,22.,23.,24.])

    ts2 = ts.TimeSeries(np.arange(200).reshape(2,100),sampling_rate=1)
    ev = ts.Events([10,20,30])
    et = nta.EventRelatedAnalyzer(ts2,ev,5)
    
    npt.assert_equal(et.eta.data,[[  20.,   21.,   22.,   23.,   24.],
                                  [ 120.,  121.,  122.,  123.,  124.]])


    #Test that providing the analyzer with an array, instead of an Events or a
    #TimeSeries object throws an error:
    npt.assert_raises(ValueError,nta.EventRelatedAnalyzer,ts2,events,10)
     
    
def test_HilbertAnalyzer():
    """Testing the HilbertAnalyzer (analytic signal)"""
    pi = np.pi
    Fs = np.pi
    t = np.arange(0,2*pi,pi/256)

    a0 = np.sin(t)
    a1 = np.cos(t)
    a2 = np.sin(2*t)
    a3 = np.cos(2*t)

    T = ts.TimeSeries(data=np.vstack([a0,a1,a2,a3]),
                             sampling_rate=Fs)

    H = nta.HilbertAnalyzer(T)

    h_abs = H.amplitude.data
    h_angle = H.phase.data
    h_real = H.real.data
    #The real part should be equal to the original signals:
    npt.assert_almost_equal(h_real,T.data)
    #The absolute value should be one everywhere, for this input:
    npt.assert_almost_equal(h_abs,np.ones(T.data.shape))
    #For the 'slow' sine - the phase should go from -pi/2 to pi/2 in the first
    #256 bins: 
    npt.assert_almost_equal(h_angle[0,:256],np.arange(-pi/2,pi/2,pi/256))
    #For the 'slow' cosine - the phase should go from 0 to pi in the same
    #interval: 
    npt.assert_almost_equal(h_angle[1,:256],np.arange(0,pi,pi/256))
    #The 'fast' sine should make this phase transition in half the time:
    npt.assert_almost_equal(h_angle[2,:128],np.arange(-pi/2,pi/2,pi/128))
    #Ditto for the 'fast' cosine:
    npt.assert_almost_equal(h_angle[3,:128],np.arange(0,pi,pi/128))

def test_FilterAnalyzer():
    """Testing the FilterAnalyzer """
    t = np.arange(np.pi/100,10*np.pi,np.pi/100)
    fast = np.sin(50*t)+10
    slow = np.sin(10*t)-20
    fast_ts = ts.TimeSeries(data=fast,sampling_rate=np.pi)
    slow_ts = ts.TimeSeries(data=slow,sampling_rate=np.pi)
   
    #Make sure that the DC is preserved
    f_slow = nta.FilterAnalyzer(slow_ts,ub=0.6)
    npt.assert_almost_equal(f_slow.filtered_fourier.data.mean(),-20)
    f_fast = nta.FilterAnalyzer(fast_ts,lb=0.6)
    npt.assert_almost_equal(f_fast.data.mean(),10)

def test_NormalizationAnalyzer():
    """Testing the NormalizationAnalyzer """

    t1 = ts.TimeSeries(data=[[99,100,101],[99,100,101]],sampling_interval=1.)
    t2 = ts.TimeSeries(data=[[-1,0,1],[-1,0,1]],sampling_interval=1.)

    N1 = nta.NormalizationAnalyzer(t1)
    npt.assert_almost_equal(N1.percent_change[0],t2[0])

    t3 = ts.TimeSeries(data=[[100,102],[1,3]],sampling_interval=1.)
    t4 = ts.TimeSeries(data=[[-1,1],[-1,1]],sampling_interval=1.)
    
    N2 = nta.NormalizationAnalyzer(t3)
    npt.assert_almost_equal(N2.z_score[0],t4[0])
    
def test_MorletWaveletAnalyzer():
    """Testing the MorletWaveletAnalyzer """
    time_series = ts.TimeSeries(data=np.random.rand(100),sampling_rate=100)

    W = nta.MorletWaveletAnalyzer(time_series,freqs=20)
    WL = nta.MorletWaveletAnalyzer(time_series,freqs=20,log_morlet=True)
    H = nta.HilbertAnalyzer(W.real)
    HL = nta.HilbertAnalyzer(WL.real)

    npt.assert_almost_equal(np.sin(H.phase.data[10:-10]),np.sin(W.phase.data[10:-10]),decimal=0)
    npt.assert_almost_equal(np.sin(HL.phase.data[10:-10]),np.sin(WL.phase.data[10:-10]),decimal=0)

#def test_EventTriggeredAnalyzer():



"""This illustrates multi-taper spectral density function estimation.

References:

[1] D. Thomson, "Spectrum estimation and harmonic analysis," Proceedings of the
IEEE, vol. 70, 1982.

[2] D.J. Thomson, "Jackknifing Multitaper Spectrum Estimates [Identifying
variances of complicated estimation procedures]," IEEE Signal Processing
Magazine, 2007, pp. 20-30.
"""

import numpy as np
import matplotlib.pyplot as pp
import scipy.stats.distributions as dist

import nitime.algorithms as alg
import nitime.utils as utils

def dB(x, out=None):
    if out is None:
        return 10 * np.log10(x)
    else:
        np.log10(x, out)
        np.multiply(out, 10, out)

### Log-to-dB conversion factor ###
ln2db = dB(np.e)

N = 512
import os
if os.path.exists('example_arrs.npz'):
    foo = np.load('example_arrs.npz')
    ar_seq = foo['arr_0']
    nz = foo['arr_1']
    alpha = foo['arr_2']
else:
    ar_seq, nz, alpha = utils.ar_generator(N=N, drop_transients=10)
    ar_seq -= ar_seq.mean()
    np.savez('example_arrs', ar_seq, nz, alpha)
# --- True SDF
fgrid, hz = alg.my_freqz(1.0, a=np.r_[1, -alpha], Nfreqs=N)
sdf = (hz*hz.conj()).real
# onesided spectrum, so double the power
sdf[1:-1] *= 2
dB(sdf, sdf)

# --- Direct Spectral Estimator
freqs, d_sdf = alg.periodogram(ar_seq)
dB(d_sdf, d_sdf)

# --- Welch's Overlapping Periodogram Method via mlab
mlab_sdf, mlab_freqs = pp.mlab.psd(ar_seq, NFFT=N)
mlab_freqs *= (np.pi/mlab_freqs.max())
mlab_sdf = mlab_sdf.squeeze()
dB(mlab_sdf, mlab_sdf)


# --- Regular Multitaper Estimate
f, sdf_mt, nu = alg.multi_taper_psd(
    ar_seq, adaptive=False, jackknife=False
    )
dB(sdf_mt, sdf_mt)
# OK.. grab the number of tapers used from here
Kmax = nu[0]/2

# --- Adaptively Weighted Multitapter Estimate
# -- Adaptive weighting from Thompson 1982, or Percival and Walden 1993
f, adaptive_sdf_mt, nu = alg.multi_taper_psd(
    ar_seq,  adaptive=True, jackknife=False
    )
dB(adaptive_sdf_mt, adaptive_sdf_mt)

# --- Jack-knifed intervals for regular weighting-----------------------------
# currently returns log-variance
_, _, jk_var = alg.multi_taper_psd(
    ar_seq, adaptive=False, jackknife=True
    )

# the Jackknife mean is approximately distributed about the true log-sdf
# as a Student's t distribution with variance jk_var ... but in
# fact the jackknifed variance better describes the normal
# multitaper estimator < have ref for this >

# find 95% confidence limits from inverse of t-dist CDF
jk_p = (dist.t.ppf(.975, Kmax-1) * np.sqrt(jk_var)) * ln2db

jk_limits = ( sdf_mt - jk_p, sdf_mt + jk_p )

# --- Jack-knifed intervals for adaptive weighting----------------------------
_, _, adaptive_jk_var = alg.multi_taper_psd(
    ar_seq, adaptive=True, jackknife=True
    )

# find 95% confidence limits from inverse of t-dist CDF
jk_p = (dist.t.ppf(.975, Kmax-1)*np.sqrt(adaptive_jk_var)) * ln2db

adaptive_jk_limits = ( adaptive_sdf_mt - jk_p, adaptive_sdf_mt + jk_p )

# --- Hypothetical intervals with chi2(2Kmax) --------------------------------
# from Percival and Walden eq 258
p975 = dist.chi2.ppf(.975, 2*Kmax)
p025 = dist.chi2.ppf(.025, 2*Kmax)

l1 = ln2db * np.log(2*Kmax/p975)
l2 = ln2db * np.log(2*Kmax/p025)

hyp_limits = ( sdf_mt + l1, sdf_mt + l2 )

# --- Hypothetical intervals with chi2(nu(f)) --------------------------------

## p975 = dist.chi2.ppf(.975, nu_f)
## p025 = dist.chi2.ppf(.025, nu_f)

## l1 = ln2db * np.log(nu_f/p975)
## l2 = ln2db * np.log(nu_f/p025)

## adaptive_hyp_limits = ( adaptive_sdf_mt + l1, adaptive_sdf_mt + l2 )

# --- Plotting ---------------------------------------------------------------
ax_limits = 2*sdf.min(), 1.25*sdf.max()

def plot_estimate(ax, f, sdf_ests, limits=None, elabels=()):
    ax.plot(f, sdf, 'c', label='True S(f)')
    if not elabels:
        elabels = ('',) * len(sdf_ests)
    colors = 'bgkmy'
    for e, l, c in zip(sdf_ests, elabels, colors):
        ax.plot(f, e, color=c, linewidth=2, label=l)

    if limits is not None:
        ax.fill_between(f, limits[0], y2=limits[1], color=(1,0,0,.3))
    ax.set_ylim(ax_limits)
    ax.legend()

f = pp.figure()
ax = f.add_subplot(611)
plot_estimate(ax, freqs, (d_sdf,), elabels=("Periodogram",))
ax = f.add_subplot(612)
plot_estimate(ax, mlab_freqs, (mlab_sdf,), elabels=("Welch's method",))
ax = f.add_subplot(613)
plot_estimate(ax, freqs, (sdf_mt,), hyp_limits,
              elabels=('MT with hypothetical 5% interval',))
ax = f.add_subplot(614)
plot_estimate(ax, freqs, (sdf_mt,),
              jk_limits,
              elabels=('MT with JK 5% interval',))
ax = f.add_subplot(615)
## plot_estimate(ax, freqs, (adaptive_sdf_mt,),
##               adaptive_hyp_limits,
##               elabels=('(a)MT with hypothetical 5% interval',))
## ax = f.add_subplot(616)
plot_estimate(ax, freqs, (adaptive_sdf_mt, ),
              adaptive_jk_limits,
              elabels=('(a)MT with JK 5% interval',))
f.text(.5, .9, '%d Tapers'%Kmax)
pp.show()



import nitime.timeseries as ts
import nitime.analysis as ta 
import nitime.viz as tv
from matplotlib.mlab import csv2rec

#Load the data from file:
data =csv2rec('data/event_related_fmri.csv')
#Initialize TimeSeries objects: 
t1 = ts.TimeSeries(data.bold,sampling_interval=2)
t2 = ts.TimeSeries(data.events,sampling_interval=2)
#Initialized the event-related analyzer with the two time-series:
E = ta.EventRelatedAnalyzer(t1,t2,15,offset=-5)
#Visualize the results:
fig = tv.plot_tseries(E.eta,ylabel='BOLD (% signal change)',yerror=E.ets)





"""nitime version/release information"""

ISRELEASED = True

# Format expected by setup.py and doc/source/conf.py: string of form "X.Y.Z"
_version_major = 0
_version_minor = 1
_version_micro = 0
__version__ = "%s.%s.%s" % (_version_major, _version_minor, _version_micro)


CLASSIFIERS = ["Development Status :: 3 - Alpha",
               "Environment :: Console",
               "Intended Audience :: Science/Research",
               "License :: OSI Approved :: BSD License",
               "Operating System :: OS Independent",
               "Programming Language :: Python",
               "Topic :: Scientific/Engineering"]

description = "Nitime: timeseries analysis for neuroscience data"

# Note: this long_description is actually a copy/paste from the top-level
# README.txt, so that it shows up nicely on PyPI.  So please remember to edit
# it only in one place and sync it correctly.
long_description = """
===================================================
 Nitime: timeseries analysis for neuroscience data
===================================================

Nitime contains a core of numerical algorithms for time-series analysis both in
the time and spectral domains, a set of container objects to represent
time-series, and auxiliary objects that expose a high level interface to the
numerical machinery and make common analysis tasks easy to express with compact
and semantically clear code.

Website
=======

Current information can always be found at the NIPY website is located
here::

    http://nipy.org/nitime

Mailing Lists
=============

Please see the developer's list here::

    http://mail.scipy.org/mailman/listinfo/nipy-devel

Code
====

You can find our sources and single-click downloads:

* `Main repository`_ on Github.
* Documentation_ for all releases and current development tree.
* Download as a tar/zip file the `current trunk`_.
* Downloads of all `available releases`_.

.. _main repository: http://github.com/nipy/nitime
.. _Documentation: http://nipy.org/nitime
.. _current trunk: http://github.com/nipy/nitime/archives/master
.. _available releases: http://github.com/nipy/nitime/downloads

       
License information
===================

Nitime is licensed under the terms of the simplified BSD license. See the file
"LICENSE" for information on the history of this software, terms & conditions
for usage, and a DISCLAIMER OF ALL WARRANTIES.

All trademarks referenced herein are property of their respective holders.

Copyright (c) 2006-2010, NIPY Developers
All rights reserved.
"""

NAME                = "nitime"
MAINTAINER          = "Nipy Developers"
MAINTAINER_EMAIL    = "nipy-devel@neuroimaging.scipy.org"
DESCRIPTION         = description
LONG_DESCRIPTION    = long_description
URL                 = "http://nipy.org/nitime"
DOWNLOAD_URL        = "http://github.com/nipy/nitime/downloads"
LICENSE             = "Simplified BSD"
AUTHOR              = "Nitime developers"
AUTHOR_EMAIL        = "nipy-devel@neuroimaging.scipy.org"
PLATFORMS           = "OS Independent"
MAJOR               = _version_major
MINOR               = _version_minor
MICRO               = _version_micro
VERSION             = __version__
PACKAGES            = ['nitime', 'nitime.fmri', 'nitime.fmri.tests',
                       'nitime.tests']
PACKAGE_DATA        = {"nitime": ["LICENSE", "tests/*.txt"]}
REQUIRES            = ["numpy", "matplotlib", "scipy"]



import numpy as np
import numpy.testing as npt
from nitime import utils as ut
import nitime.timeseries as ts
import nose.tools as nt
import decotest

@decotest.parametric
def test_TimeArray():

    time1 = ts.TimeArray(range(100),time_unit='ms')
    time2 = time1+time1
    yield npt.assert_equal(time2.time_unit,'ms')
    time1 = ts.TimeArray(10**6)
    yield npt.assert_equal(time1.__repr__(),'1000000.0 s')
    #TimeArray can't be more than 1-d:
    yield nt.assert_raises, ValueError, ts.Events, np.zeros((2,2))
    

def test_TimeArray_init_int64():
    """Make sure that we can initialize TimeArray with an array of ints"""
    time = ts.TimeArray(np.int64(1))
    npt.assert_equal(time.__repr__(), '1.0 s')

    pass


def test_TimeArray_repr():
    """
>>> a = ts.TimeArray([1.1,2,3])
>>> a
TimeArray([ 1.1,  2. ,  3. ], time_unit='s')
>>> t = ts.TimeArray(a,time_unit='ms')
>>> t
TimeArray([ 1100.,  2000.,  3000.], time_unit='ms')
>>> t[0]
1100.0 ms

    """

@decotest.parametric
def test_TimeArray_copyflag():
    """Testing the setting of the copy-flag, where that makes sense""" 

    #These two should both generate a TimeArray, with one picosecond.
    #This one holds time_unit='s'
    t1 = ts.TimeArray(np.array([1],dtype=np.int64),copy=False)
    #This one holds time_unit='ps':
    t2 = ts.TimeArray(1,time_unit='ps')
    t3 = ts.TimeArray(t2,copy=False)
    
    yield npt.assert_equal(t1,t2)
    yield npt.assert_equal(t2.ctypes.data,t3.ctypes.data)
    
@decotest.parametric
def test_TimeArray_new():
    for unit in ['ns','ms','s',None]:
        for flag,assertion in [(True,nt.assert_not_equal),
                (False, nt.assert_equal)]:
            #list -doesn't make sense to set copy=True
            time2 = ts.TimeArray(range(5), time_unit=unit, copy=True)
            #numpy array (float) - doesn't make sense to set copy=True
            time2f = ts.TimeArray(np.arange(5.), time_unit=unit, copy=True)
            #TimeArray
            time3 = ts.TimeArray(time2, time_unit=unit, copy=flag)
            #integer
            time4 = ts.TimeArray(5,time_unit=unit,copy=True)
            #float
            time5 = ts.TimeArray(5.0,time_unit=unit,copy=True)

            yield npt.assert_equal(time2,time2f)
            yield npt.assert_equal(time2,time3)
            time3[0] +=100
            yield assertion(time2[0],time3[0])
            yield npt.assert_equal(time2[1:],time3[1:])
            yield npt.assert_equal(time4,time5)

@decotest.parametric
def test_TimeArray_bool():
    time1 = ts.TimeArray([1,2,3],time_unit='s')
    time2 = ts.TimeArray([1000,2000,3000],time_unit='ms')
    bool_arr = np.ones(time1.shape,dtype=bool)
    yield npt.assert_equal(time1,time2)
    yield npt.assert_equal(bool_arr,time1==time2)
    yield nt.assert_not_equal(type(time1==time2),ts.TimeArray)


def test_TimeArray_convert_unit():
    """
    >>> a = ts.TimeArray([1,2,3,4])
    >>> a.convert_unit('ms')
    >>> a
    TimeArray([ 1000.,  2000.,  3000.,  4000.], time_unit='ms')
    >>> a.time_unit
    'ms'
    >>> b = ts.TimeArray([1,2,3,4],'s')
    >>> a==b
    array([ True,  True,  True,  True], dtype=bool)
    """
    

@decotest.parametric
def test_TimeArray_div():

    #divide singelton by singleton:
    a = 2.0
    b = 6.0
    time1 = ts.TimeArray(a,time_unit='s')
    time2 = ts.TimeArray(b,time_unit='s')
    div1 = a/b
    #This should eliminate the units and return a float, not a TimeArray:
    div2 = time1/time2
    yield npt.assert_equal(div1,div2)

    #Divide a TimeArray by a singelton:
    a = np.array([1,2,3])
    b = 6.0
    time1 = ts.TimeArray(a,time_unit='s')
    time2 = ts.TimeArray(b,time_unit='s')
    div1 = a/b
    #This should eliminate the units and return a float array, not a TimeArray:
    div2 = time1/time2
    yield npt.assert_equal(div1,div2)

    #Divide a TimeArray by another TimeArray:
    a = np.array([1,2,3])
    b = np.array([2,2,2]).astype(float) #TimeArray division is float division!
    time1 = ts.TimeArray(a,time_unit='s')
    time2 = ts.TimeArray(b,time_unit='s')
    div1 = a/b
    #This should eliminate the units and return a float array, not a TimeArray:
    div2 = time1/time2
    yield npt.assert_equal(div1,div2)
    
@decotest.parametric
def test_TimeArray_index_at():
    time1 = ts.TimeArray(range(10),time_unit='ms')
    for i in xrange(10):
        idx = time1.index_at([i])
        yield npt.assert_equal(idx,np.array(i))
        idx_secs=time1.index_at(ts.TimeArray(i/1000.))
        yield npt.assert_equal(idx_secs,np.array(i))

@decotest.parametric
def test_TimeArray_at():
    time1 = ts.TimeArray(range(10),time_unit='ms')
    for i in xrange(10):
        this = time1.at(i)
        yield npt.assert_equal(this,ts.TimeArray(i,time_unit='ms'))
        this_secs=time1.at(ts.TimeArray(i/1000.))
        yield npt.assert_equal(this_secs,ts.TimeArray(i,time_unit='ms'))
        seconds_array = ts.TimeArray(time1,time_unit='s')
        this_secs=seconds_array.at(i/1000.)
        yield npt.assert_equal(this_secs,ts.TimeArray(i,time_unit='ms'))
        all=time1.at(ts.TimeArray(i/1000.), tol=10)
        yield npt.assert_equal(all,time1)
        if i>0 and i<9:
            this_secs=time1.at(ts.TimeArray(i/1000.), tol=1)
            yield npt.assert_equal(this_secs,ts.TimeArray([i-1,i,i+1],time_unit='ms'))

@decotest.parametric
def test_UniformTime_index_at():
    time1 = ts.UniformTime(t0=1000,length=10, sampling_rate=1000, time_unit='ms')
    mask = [False]*10
    for i in xrange(10):
        idx = time1.index_at(ts.TimeArray(1000+i,time_unit='ms'))
        yield npt.assert_equal(idx,np.array(i))
        mask[i]=True
        mask_idx = time1.index_at(ts.TimeArray(1000+i,time_unit='ms'),boolean=True)
        yield npt.assert_equal(mask_idx,mask)
        if i>0 and i<9:
            mask[i-1]=True
            mask[i+1]=True

            mask_idx = time1.index_at( ts.TimeArray([999+i,1000+i, 1001+i],
                time_unit='ms'),boolean=True)
            yield npt.assert_equal(mask_idx,mask)
        
            mask[i-1]=False
            mask[i+1]=False
        mask[i]=False

#XXX Need to write these tests:

#Test the unit conversion:
#@decotest.parametric
#def test_TimeArray_unit_conversion():

#Test the overloaded __getitem__ and __setitem: 
#@decotest.parametric
#def test_TimeArray_getset():

@decotest.parametric
def test_UniformTime():
    tuc = ts.time_unit_conversion
    
    for unit,duration in zip(['ns','ms','s',None],
                             [2*10**9,2*10**6,100,20]):
        
        t1 = ts.UniformTime(duration=duration,sampling_rate=1,
                            time_unit=unit)
        t2 = ts.UniformTime(duration=duration,sampling_rate=20,
                            time_unit=unit)

        #The following two tests verify that first-last are equal to the
        #duration, but it is unclear whether that is really the behavior we
        #want, because the t_i held by a TimeSeries is the left
        #(smaller) side of the time-duration defined by the bin
        
        #The difference between the first and last item is the duration:
        #yield npt.assert_equal(t1[-1]-t1[0],
        #                       ts.TimeArray(duration,time_unit=unit))
        #Duration doesn't depend on the sampling rate:
        #yield npt.assert_equal(t1[-1]-t2[0],
        #                       ts.TimeArray(duration,time_unit=unit))

        a = ts.UniformTime(duration=10,sampling_rate=1)
        b = ts.UniformTime(a,time_unit=unit)
        yield npt.assert_equal(a.sampling_interval,b.sampling_interval)
        yield npt.assert_equal(a.sampling_rate,b.sampling_rate)

        b = ts.UniformTime(a,duration=2*duration,time_unit=unit)
        yield npt.assert_equal(a.sampling_interval,b.sampling_interval)
        yield npt.assert_equal(a.sampling_rate,b.sampling_rate)
            
        b = ts.UniformTime(a,length=100,time_unit=unit)
        yield npt.assert_equal(a.sampling_interval,b.sampling_interval)
        yield npt.assert_equal(a.sampling_rate,b.sampling_rate)

        b = ts.UniformTime(a,length=100,time_unit=unit)
        yield npt.assert_equal(a.sampling_interval,b.sampling_interval)
        yield npt.assert_equal(a.sampling_rate,b.sampling_rate)
        
        b = ts.UniformTime(a,length=100,duration=duration,time_unit=unit)
        c = ts.UniformTime(length=100,duration=duration,time_unit=unit)
        yield npt.assert_equal(c,b)

        
        b = ts.UniformTime(sampling_interval=1,duration=10,time_unit=unit)
        c = ts.UniformTime(sampling_rate=tuc['s']/tuc[unit],
                           length=10,time_unit=unit)

        yield npt.assert_equal(c,b)

        #This should raise a value error, because the duration is shorter than
        #the sampling_interval:
        yield npt.assert_raises(ValueError,
                          ts.UniformTime,dict(sampling_interval=10,duration=1))

    #Time objects can be initialized with other time objects setting the
    #duration, sampling_interval and sampling_rate:
    
    a = ts.UniformTime(length=1,sampling_rate=1)
    yield npt.assert_raises(ValueError, ts.UniformTime, dict(data=a,
        sampling_rate=10, sampling_interval=.1))
    b = ts.UniformTime(duration=2*a.sampling_interval,
                       sampling_rate=2*a.sampling_rate)

    yield npt.assert_equal(ts.Frequency(b.sampling_rate),
                     ts.Frequency (2*a.sampling_rate))
    yield npt.assert_equal(b.sampling_interval,ts.TimeArray(0.5*a.sampling_rate))

    b = ts.UniformTime(duration=10,
                       sampling_interval=a.sampling_interval)

    yield npt.assert_equal(b.sampling_rate,a.sampling_rate)

    b = ts.UniformTime(duration=10,
                       sampling_rate=a.sampling_rate)

    yield npt.assert_equal(b.sampling_interval,a.sampling_interval)


def test_UniformTime_repr():
    """
    >>> time1 = ts.UniformTime(sampling_rate=1000,time_unit='ms',length=3)
    >>> time1.sampling_rate
    1000.0 Hz
    >>> time1
    UniformTime([ 0.,  1.,  2.], time_unit='ms')

    >>> time2= ts.UniformTime(sampling_rate=1000,time_unit='s',length=3)
    >>> time2.sampling_rate
    1000.0 Hz
    >>> time2
    UniformTime([ 0.   ,  0.001,  0.002], time_unit='s')

    >>> a = ts.UniformTime(length=5,sampling_rate=1,time_unit='ms')

    >>> b = ts.UniformTime(a)

    >>> b
    UniformTime([    0.,  1000.,  2000.,  3000.,  4000.], time_unit='ms')

    >>> a
    UniformTime([    0.,  1000.,  2000.,  3000.,  4000.], time_unit='ms')

    >>> b = ts.UniformTime(a,time_unit='s')

    >>> b
    UniformTime([ 0.,  1.,  2.,  3.,  4.], time_unit='s')

    >>> a = ts.UniformTime(length=1,sampling_rate=2)

    >>> b = ts.UniformTime(length=10,sampling_interval=a.sampling_interval)

    >>> b.sampling_rate
    2.0 Hz
    """

@decotest.parametric
def test_Frequency():
    """Test frequency representation object"""
    tuc=ts.time_unit_conversion
    for unit in ['ns','ms','s',None]:
        f = ts.Frequency(1,time_unit=unit)
        yield npt.assert_equal(f.to_period(),tuc[unit]) 

        f = ts.Frequency(1000,time_unit=unit)
        yield npt.assert_equal(f.to_period(),tuc[unit]/1000)       

        f = ts.Frequency(0.001,time_unit=unit)
        yield npt.assert_equal(f.to_period(),tuc[unit]*1000)       

    
@decotest.parametric
def test_TimeSeries():
    """Testing the initialization of the uniform time series object """ 

    #Test initialization with duration:
    tseries1 = ts.TimeSeries([1,2,3,4,5,6,7,8,9,10],duration=10)
    tseries2 = ts.TimeSeries([1,2,3,4,5,6,7,8,9,10],sampling_interval=1)
    npt.assert_equal(tseries1.time,tseries2.time)

    #downsampling:
    t1 = ts.UniformTime(length=8,sampling_rate=2)
    #duration is the same, but we're downsampling to 1Hz
    tseries1 = ts.TimeSeries(data=[1,2,3,4],time=t1,sampling_rate=1)
    #If you didn't explicitely provide the rate you want to downsample to, that
    #is an error:
    yield npt.assert_raises(ValueError,ts.TimeSeries,dict(data=[1,2,3,4],
                                                           time=t1)) 

    tseries2 = ts.TimeSeries(data=[1,2,3,4],sampling_rate=1)
    tseries3 = ts.TimeSeries(data=[1,2,3,4],sampling_rate=1000,
                                    time_unit='ms')
    #you can specify the sampling_rate or the sampling_interval, to the same
    #effect, where specificying the sampling_interval is in the units of that
    #time-series: 
    tseries4 = ts.TimeSeries(data=[1,2,3,4],sampling_interval=1,
                                        time_unit='ms')
    yield npt.assert_equal(tseries4.time,tseries3.time)

    #The units you use shouldn't matter - time is time:
    tseries6 = ts.TimeSeries(data=[1,2,3,4],
                                    sampling_interval=0.001,
                                    time_unit='s')
    yield npt.assert_equal(tseries6.time,tseries3.time)

    #And this too - perverse, but should be possible: 
    tseries5 = ts.TimeSeries(data=[1,2,3,4],
                                    sampling_interval=ts.TimeArray(0.001,
                                                         time_unit='s'),
                                    time_unit='ms')

    yield npt.assert_equal(tseries5.time,tseries3.time)

    #initializing with a UniformTime object:
    t=ts.UniformTime(length=3,sampling_rate=3)

    data = [1,2,3]

    tseries7 = ts.TimeSeries(data=data,time=t)

    yield npt.assert_equal(tseries7.data,data)

    data = [1,2,3,4]
    #If the data is not the right length, that should throw an error:
    yield npt.assert_raises(ValueError,
                          ts.TimeSeries,dict(data=data,time=t))
    
def test_TimeSeries_repr():

    """
    >>> t=ts.UniformTime(length=3,sampling_rate=3)
    >>> tseries1 = ts.TimeSeries(data=[3,5,8],time=t)
    >>> t.sampling_rate
    3.0 Hz
    >>> tseries1.sampling_rate
    3.0 Hz
    >>> tseries1 = ts.TimeSeries(data=[3,5,8],sampling_rate=3)
    >>> tseries1.time
    UniformTime([ 0.        ,  0.33333333,  0.66666667], time_unit='s')
    >>> tseries1.sampling_rate
    3.0 Hz
    >>> tseries1.sampling_interval
    0.33333333333300003 s
    >>> a = ts.UniformTime(length=1,sampling_rate=2)

    >>> b = ts.TimeSeries(data=[1,2,3],sampling_interval=a.sampling_interval)

    >>> b.sampling_rate
    2.0 Hz


    >>> a = ts.UniformTime(length=1,sampling_rate=1)

    >>> b = ts.TimeSeries(data=[1,2,3],sampling_interval=a.sampling_interval)

    >>> b.sampling_rate
    1.0 Hz
    """ 
    
@decotest.parametric
def test_Epochs():
    tms = ts.TimeArray(data=range(100), time_unit='ms')
    tmin = ts.TimeArray(data=range(100), time_unit='m')
    tsec = ts.TimeArray(data=range(100), time_unit='s')
    
    utms = ts.UniformTime(length=100, sampling_interval=1, time_unit='ms')
    utmin = ts.UniformTime(length=100, sampling_interval=1, time_unit='m')
    utsec = ts.UniformTime(length=100, sampling_interval=1, time_unit='s')

    tsms = ts.TimeSeries(data=range(100), sampling_interval=1, time_unit='ms')
    tsmin = ts.TimeSeries(data=range(100), sampling_interval=1, time_unit='m')
    tssec = ts.TimeSeries(data=range(100), sampling_interval=1, time_unit='s')

    # one millisecond epoch
    e1ms = ts.Epochs(0,1, time_unit='ms')
    msg = "Seems like a problem with copy=False in TimeArray constructor."
    yield npt.assert_equal(e1ms.duration, ts.TimeArray(1,time_unit='ms'),msg)


    # one day
    e1d = ts.Epochs(0,1, time_unit='D')
    yield npt.assert_equal(e1d.duration, ts.TimeArray(1,time_unit='D'),msg)

    e1ms_ar = ts.Epochs([0,0],[1,1], time_unit='ms')

    for t in [tms, tmin, tsec, utms, utmin, utsec]:
        # the sample time arrays are all at least 1ms long, so this should
        # return a timearray that has exactly one time point in it
        yield npt.assert_equal(len(t.during(e1ms)),1)

        # make sure, slicing doesn't change the class
        yield npt.assert_equal(type(t),type(t.during(e1ms)))
        
    for t in [tsms, tsmin, tssec]:
        # the sample time series are all at least 1ms long, so this should
        # return a timeseries that has exactly one time point in it
        yield npt.assert_equal(len(t.during(e1ms)),1)
        
        # make sure, slicing doesn't change the class
        yield npt.assert_equal(type(t),type(t.during(e1ms)))

        # same thing but now there's an array of epochs
        e2 = ts.Epochs([0,10],[10,20],time_unit=t.time_unit)
        
        # make sure, slicing doesn't change the class for array of epochs
        yield npt.assert_equal(type(t),type(t.during(e2)))

        # Indexing with an array of epochs (all of which are the same length)
        yield npt.assert_equal(t[e2].data.shape, (2,10))
        yield npt.assert_equal(len(t.during(e2)),10)
        yield npt.assert_equal(t[e2].data.ndim,2)
        # check the data at some timepoints (a dimension was added)
        yield npt.assert_equal(t[e2][0], (0,10))
        yield npt.assert_equal(t[e2][1], (1,11))
        # check the data for each epoch
        yield npt.assert_equal(t[e2].data[0], range(10))
        yield npt.assert_equal(t[e2].data[1], range(10,20))
        yield npt.assert_equal(t[e2].duration,e2[0].duration)
        
        # slice with Epochs of different length (not supported for timeseries,
        # raise error, though future jagged array implementation could go here)
        ejag = ts.Epochs([0,10],[10,40],time_unit=t.time_unit)
        # next line is the same as t[ejag]
        yield npt.assert_raises(ValueError,t.__getitem__, ejag)


        # if an epoch lies entirely between samples in the timeseries, return
        # the previous sample. We define the bin as any time between
        # [t,t+delta_t)
        eshort = ts.Epochs(2.5,2.7,time_unit=t.time_unit)
        yield npt.assert_equal(len(t[eshort].data),1)
        yield npt.assert_equal(t[eshort].data[0],t[2])

        e1ms_outofrange = ts.Epochs(200,300,time_unit=t.time_unit)
        # assert that with the epoch moved outside of the time range of our
        # data, slicing with the epoch now yields an empty array
        yield npt.assert_raises(ValueError, t.during, dict(e=e1ms_outofrange))

        # the sample timeseries are all shorter than a day, so this should
        # raise an error (instead of padding, or returning a shorter than
        # expected array.
        yield npt.assert_raises(ValueError, t.during, dict(e=e1d))


@decotest.parametric
def test_Events():

    # time has to be one-dimensional
    yield nt.assert_raises, ValueError, ts.Events, np.zeros((2,2))

    t = ts.TimeArray([1,2,3],time_unit='ms')
    x = [1,2,3]
    y = [2,4,6]
    z = [10.,20.,30.]
    i0 = [0,0,1]
    i1 = [0,1,2]
    for unit in [None, 's','ns','D']:
        # events with data
        ev1 = ts.Events(t,time_unit=unit,i=x,j=y,k=z)

        # events with indices
        ev2 = ts.Events(t,time_unit=unit,indices=[i0,i1])

        # events with indices and labels
        ev3 = ts.Events(t,time_unit=unit,labels=['trial','other'],
                        indices=[i0,i1])

        # Note that the length of indices and labels has to be identical: 
        yield nt.assert_raises, ValueError,ts.Events,dict(t=t,time_unit=unit,labels=['trial','other'],indices=[i0])#Only one of the indices!
            

        # make sure the time is retained
        yield npt.assert_equal(ev1.time,t)
        yield npt.assert_equal(ev2.time,t)

        # make sure time unit is correct
        if unit is not None:
            yield npt.assert_equal(ev1.time_unit,unit)
            yield npt.assert_equal(ev2.time_unit,unit)
        else:
            yield npt.assert_equal(ev1.time_unit,'s')
            yield npt.assert_equal(ev2.time_unit,'s')

        # make sure we can extract data
        yield npt.assert_equal(ev1.data['i'],x)
        yield npt.assert_equal(ev1.data['j'],y)
        yield npt.assert_equal(ev1.data['k'],z)

        # make sure we can get the indices by label
        yield npt.assert_equal(ev3.index.trial,i0)
        yield npt.assert_equal(ev3.index.other,i1)

        # make sure we can get the indices by position
        yield npt.assert_equal(ev2.index.i0,i0)
        yield npt.assert_equal(ev2.index.i1,i1)


    



import numpy as np
import numpy.testing as npt
import nitime.timeseries as ts
import decotest
import nose.tools as nt

from scipy.signal import signaltools

from nitime import utils

def test_zscore():

    x = np.array([[1,2,3],[4,5,6]])
    z = utils.zscore(x)

    npt.assert_equal(x.shape,z.shape)
    
def test_percent_change():
    x = np.array([[99,100,101],[4,5,6]])
    p = utils.percent_change(x)

    npt.assert_equal(x.shape,p.shape)
    npt.assert_almost_equal(p[0,2],1.0)

def test_debias():
    x = np.arange(64).reshape(4,4,4)
    x0 = utils.remove_bias(x, axis=1)
    assert (x0.mean(axis=1)==0).all(), \
           'did not remove the bias from axis 1'

def ref_crosscov(x, y, all_lags=True):
    "Computes sxy[k] = E{x[n]*y[n+k]}"
    x = utils.remove_bias(x, 0)
    y = utils.remove_bias(y, 0)
    lx, ly = len(x), len(y)
    pad_len = lx + ly - 1
    sxy = np.correlate(x, y, mode='full')/lx
    if all_lags:
        return sxy
    c_idx = pad_len/2
    return sxy[c_idx:]

def test_crosscov():
    N = 128
    ar_seq1, _, _ = utils.ar_generator(N=N)
    ar_seq2, _, _ = utils.ar_generator(N=N)

    for all_lags in (True, False):
        sxy = utils.crosscov(ar_seq1, ar_seq2, all_lags=all_lags)
        sxy_ref = ref_crosscov(ar_seq1, ar_seq2, all_lags=all_lags)
        err = sxy_ref - sxy
        mse = np.dot(err, err)/N
        yield nt.assert_true, mse < 1e-12, \
               'Large mean square error w.r.t. reference cross covariance'

def test_autocorr():
    N = 128
    ar_seq, _, _ = utils.ar_generator(N=N)
    rxx = utils.autocorr(ar_seq)
    yield nt.assert_true, rxx[0]==1, \
          'Zero lag autocorrelation is not equal to 1'
    rxx = utils.autocorr(ar_seq, all_lags=True)
    yield nt.assert_true, rxx[127]==1, \
          'Zero lag autocorrelation is not equal to 1'

def plot_savings():
    import matplotlib.pyplot as pp
    from IPython.Magic import timings
    f1 = ref_crosscov
    f2 = utils.crosscov
    times1 = []
    times2 = []
    for N in map(lambda x: 2**x, range(8,13)):
        a = np.random.randn(N)
        args = (a,a)
        kws = dict()
        _, t_est1 = timings(1000, f1, *args, **kws)
        _, t_est2 = timings(1000, f2, *args, **kws)
        times1.append(t_est1)
        times2.append(t_est2)
    pp.figure()
    pp.plot(range(8,13), times1, label='tdomain crosscov')
    pp.plot(range(8,13), times2, label='fdomain crosscov')
    pp.legend()
    pp.show()



"""Tests for our own test utilities.

This file can be run as a script, and it will call unittest.main().  We must
check that it works with unittest as well as with nose...
"""

from decotest import as_unittest, ParametricTestCase, parametric

@as_unittest
def trivial():
    """A trivial test"""
    pass

# Some examples of parametric tests.

def is_smaller(i,j):
    assert i<j,"%s !< %s" % (i,j)

class Tester(ParametricTestCase):

    def test_parametric(self):
        yield is_smaller(3, 4)
        x, y = 1, 2
        yield is_smaller(x, y)

@parametric
def test_par_standalone():
    yield is_smaller(3, 4)
    x, y = 1, 2
    yield is_smaller(x, y)


def test_par_nose():
    yield (is_smaller,3, 4)
    x, y = 2, 3
    yield (is_smaller,x, y)


if __name__ == '__main__':
    import unittest
    unittest.main()



import numpy as np
import nitime.timeseries as ts
import nitime.analysis as tsa
reload(tsa)
import nitime.viz as viz

#As before the stimuli get read from files:  
maxdB1 = 76.4286 #Taken from the spike file header
stim1 = np.loadtxt('data/grasshopper_stimulus1.txt')
stim1 = (20*1/np.log(10))*(np.log(stim1[:,1]/2.0e-5))
stim1 = maxdB1-stim1.max()+stim1

maxdB2 = 71.2 #Taken from the spike file header
stim2 = np.loadtxt('data/grasshopper_stimulus2.txt')
stim2 = (20*1/np.log(10))*(np.log(stim2[:,1]/2.0e-5))
stim2 = maxdB1-stim2.max()+stim2

#Get the spike-times:
spike_times1 = np.loadtxt('data/grasshopper_spike_times1.txt')
spike_times2 = np.loadtxt('data/grasshopper_spike_times2.txt')

et = []
means = []
for stim,spike in zip([stim1,stim2],[spike_times1,spike_times2]):
    stim_time_series = ts.TimeSeries(t0=0,data=stim,sampling_interval=50,
                                 time_unit='us')
    
    stim_time_series.time_unit = 'ms'

    spike_ev = ts.Events(spike,time_unit='us')
    #Initialize the event-related analyzer
    event_related = tsa.EventRelatedAnalyzer(stim_time_series,spike_ev,
                                             len_et=200,
                                             offset=-200)

    et.append(event_related.eta)
    means.append(np.mean(stim_time_series.data))

#Stack the data from both time-series, initialize a new time-series and 
fig =viz.plot_tseries(ts.TimeSeries(data=np.vstack([et[0].data,et[1].data]),
                               sampling_rate=et[0].sampling_rate,time_unit='ms'))

ax = fig.get_axes()[0]
xlim = ax.get_xlim()
#Now we plot the means for each of the stimulus time-series:
ax.plot([xlim[0],xlim[1]],[means[0],means[0]],'b--')
ax.plot([xlim[0],xlim[1]],[means[1],means[1]],'g--')



    
    



#!/usr/bin/env python
"""Simple example AR(p) fitting."""

import numpy as np
from matplotlib import pyplot as plt

from nitime import utils
from nitime import algorithms as alg
reload(utils)

npts = 2048*10
sigma = 1
drop_transients = 1024
coefs = np.array([0.9, -0.5])

# Generate AR(2) time series
X, v, _ = utils.ar_generator(npts, sigma, coefs, drop_transients)

# Visualize
plt.figure()
plt.plot(v)
plt.title('noise')
plt.figure()
plt.plot(X)
plt.title('AR signal')

# Estimate the model parameters
sigma_est, coefs_est = alg.yule_AR_est(X, 2, 2*npts, system=True)

print 'coefs    :', coefs
print 'coefs est:', coefs_est

plt.show()



"""A special directive for including a matplotlib plot.

The source code for the plot may be included in one of two ways:

  1. A path to a source file as the argument to the directive::

       .. plot:: path/to/plot.py

     When a path to a source file is given, the content of the
     directive may optionally contain a caption for the plot::

       .. plot:: path/to/plot.py

          This is the caption for the plot

     Additionally, one my specify the name of a function to call (with
     no arguments) immediately after importing the module::

       .. plot:: path/to/plot.py plot_function1

  2. Included as inline content to the directive::

     .. plot::

        import matplotlib.pyplot as plt
        import matplotlib.image as mpimg
        import numpy as np
        img = mpimg.imread('_static/stinkbug.png')
        imgplot = plt.imshow(img)

In HTML output, `plot` will include a .png file with a link to a high-res
.png and .pdf.  In LaTeX output, it will include a .pdf.

To customize the size of the plot, this directive supports all of the
options of the `image` directive, except for `target` (since plot will
add its own target).  These include `alt`, `height`, `width`, `scale`,
`align` and `class`.

Additionally, if the `:include-source:` option is provided, the
literal source will be displayed inline in the text, (as well as a
link to the source in HTML).  If this source file is in a non-UTF8 or
non-ASCII encoding, the encoding must be specified using the
`:encoding:` option.

The set of file formats to generate can be specified with the
`plot_formats` configuration variable.
"""

import sys, os, shutil, imp, warnings, cStringIO, re
try:
    from hashlib import md5
except ImportError:
    from md5 import md5

from docutils.parsers.rst import directives
try:
    # docutils 0.4
    from docutils.parsers.rst.directives.images import align
except ImportError:
    # docutils 0.5
    from docutils.parsers.rst.directives.images import Image
    align = Image.align
import sphinx

sphinx_version = sphinx.__version__.split(".")
# The split is necessary for sphinx beta versions where the string is
# '6b1'
sphinx_version = tuple([int(re.split('[a-z]', x)[0])
                        for x in sphinx_version[:2]])

import matplotlib
import matplotlib.cbook as cbook
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.image as image
from matplotlib import _pylab_helpers
from matplotlib.sphinxext import only_directives


class PlotWarning(Warning):
    """Warning category for all warnings generated by this directive.

    By printing our warnings with this category, it becomes possible to turn
    them into errors by using in your conf.py::

      warnings.simplefilter('error', plot_directive.PlotWarning)

    This way, you can ensure that your docs only build if all your examples
    actually run successfully.
    """
    pass


# os.path.relpath is new in Python 2.6
if hasattr(os.path, 'relpath'):
    relpath = os.path.relpath
else:
    # This code is snagged from Python 2.6

    def relpath(target, base=os.curdir):
        """
        Return a relative path to the target from either the current dir or an optional base dir.
        Base can be a directory specified either as absolute or relative to current dir.
        """

        if not os.path.exists(target):
            raise OSError, 'Target does not exist: '+target

        if not os.path.isdir(base):
            raise OSError, 'Base is not a directory or does not exist: '+base

        base_list = (os.path.abspath(base)).split(os.sep)
        target_list = (os.path.abspath(target)).split(os.sep)

        # On the windows platform the target may be on a completely
        # different drive from the base.
        if os.name in ['nt','dos','os2'] and base_list[0] <> target_list[0]:
            raise OSError, 'Target is on a different drive to base. Target: '+target_list[0].upper()+', base: '+base_list[0].upper()

        # Starting from the filepath root, work out how much of the
        # filepath is shared by base and target.
        for i in range(min(len(base_list), len(target_list))):
            if base_list[i] <> target_list[i]: break
        else:
            # If we broke out of the loop, i is pointing to the first
            # differing path elements.  If we didn't break out of the
            # loop, i is pointing to identical path elements.
            # Increment i so that in all cases it points to the first
            # differing path elements.
            i+=1

        rel_list = [os.pardir] * (len(base_list)-i) + target_list[i:]
        if rel_list:
            return os.path.join(*rel_list)
        else:
            return ""

template = """
.. htmlonly::

   %(links)s

   .. figure:: %(prefix)s%(tmpdir)s/%(outname)s.png
%(options)s

%(caption)s

.. latexonly::
   .. figure:: %(prefix)s%(tmpdir)s/%(outname)s.pdf
%(options)s

%(caption)s

"""

exception_template = """
.. htmlonly::

   [`source code <%(linkdir)s/%(basename)s.py>`__]

Exception occurred rendering plot.

"""

template_content_indent = '      '

def out_of_date(original, derived):
    """
    Returns True if derivative is out-of-date wrt original,
    both of which are full file paths.
    """
    return (not os.path.exists(derived) or
            (os.path.exists(original) and
             os.stat(derived).st_mtime < os.stat(original).st_mtime))

def run_code(plot_path, function_name, plot_code):
    """
    Import a Python module from a path, and run the function given by
    name, if function_name is not None.
    """
    # Change the working directory to the directory of the example, so
    # it can get at its data files, if any.  Add its path to sys.path
    # so it can import any helper modules sitting beside it.
    if plot_code is not None:
        exec(plot_code)
    else:
        pwd = os.getcwd()
        path, fname = os.path.split(plot_path)
        sys.path.insert(0, os.path.abspath(path))
        stdout = sys.stdout
        sys.stdout = cStringIO.StringIO()
        os.chdir(path)
        fd = None
        try:
            fd = open(fname)
            module = imp.load_module(
                "__plot__", fd, fname, ('py', 'r', imp.PY_SOURCE))
        finally:
            del sys.path[0]
            os.chdir(pwd)
            sys.stdout = stdout
            if fd is not None:
                fd.close()

        if function_name is not None:
            getattr(module, function_name)()

def run_savefig(plot_path, basename, tmpdir, destdir, formats):
    """
    Once a plot script has been imported, this function runs savefig
    on all of the figures in all of the desired formats.
    """
    fig_managers = _pylab_helpers.Gcf.get_all_fig_managers()
    for i, figman in enumerate(fig_managers):
        for j, (format, dpi) in enumerate(formats):
            if len(fig_managers) == 1:
                outname = basename
            else:
                outname = "%s_%02d" % (basename, i)
            outname = outname + "." + format
            outpath = os.path.join(tmpdir, outname)
            try:
                figman.canvas.figure.savefig(outpath, dpi=dpi)
            except:
                s = cbook.exception_to_str("Exception saving plot %s" % plot_path)
                warnings.warn(s, PlotWarning)
                return 0
            if j > 0:
                shutil.copyfile(outpath, os.path.join(destdir, outname))

    return len(fig_managers)

def clear_state():
    plt.close('all')
    matplotlib.rcdefaults()
    # Set a default figure size that doesn't overflow typical browser
    # windows.  The script is free to override it if necessary.
    matplotlib.rcParams['figure.figsize'] = (5.5, 4.5)

def render_figures(plot_path, function_name, plot_code, tmpdir, destdir,
                   formats):
    """
    Run a pyplot script and save the low and high res PNGs and a PDF
    in outdir.
    """
    plot_path = str(plot_path)  # todo, why is unicode breaking this
    basedir, fname = os.path.split(plot_path)
    basename, ext = os.path.splitext(fname)

    all_exists = True

    # Look for single-figure output files first
    for format, dpi in formats:
        outname = os.path.join(tmpdir, '%s.%s' % (basename, format))
        if out_of_date(plot_path, outname):
            all_exists = False
            break

    if all_exists:
        return 1

    # Then look for multi-figure output files, assuming
    # if we have some we have all...
    i = 0
    while True:
        all_exists = True
        for format, dpi in formats:
            outname = os.path.join(
                tmpdir, '%s_%02d.%s' % (basename, i, format))
            if out_of_date(plot_path, outname):
                all_exists = False
                break
        if all_exists:
            i += 1
        else:
            break

    if i != 0:
        return i

    # We didn't find the files, so build them

    clear_state()
    try:
        run_code(plot_path, function_name, plot_code)
    except:
        s = cbook.exception_to_str("Exception running plot %s" % plot_path)
        warnings.warn(s, PlotWarning)
        return 0

    num_figs = run_savefig(plot_path, basename, tmpdir, destdir, formats)

    if '__plot__' in sys.modules:
        del sys.modules['__plot__']

    return num_figs

def _plot_directive(plot_path, basedir, function_name, plot_code, caption,
                    options, state_machine):
    formats = setup.config.plot_formats
    if type(formats) == str:
        formats = eval(formats)

    fname = os.path.basename(plot_path)
    basename, ext = os.path.splitext(fname)

    # Get the directory of the rst file, and determine the relative
    # path from the resulting html file to the plot_directive links
    # (linkdir).  This relative path is used for html links *only*,
    # and not the embedded image.  That is given an absolute path to
    # the temporary directory, and then sphinx moves the file to
    # build/html/_images for us later.
    rstdir, rstfile = os.path.split(state_machine.document.attributes['source'])
    outdir = os.path.join('plot_directive', basedir)
    reldir = relpath(setup.confdir, rstdir)
    linkdir = os.path.join(reldir, outdir)

    # tmpdir is where we build all the output files.  This way the
    # plots won't have to be redone when generating latex after html.

    # Prior to Sphinx 0.6, absolute image paths were treated as
    # relative to the root of the filesystem.  0.6 and after, they are
    # treated as relative to the root of the documentation tree.  We
    # need to support both methods here.
    tmpdir = os.path.join('build', outdir)
    tmpdir = os.path.abspath(tmpdir)
    if sphinx_version < (0, 6):
        prefix = ''
    else:
        prefix = '/'
    if not os.path.exists(tmpdir):
        cbook.mkdirs(tmpdir)

    # destdir is the directory within the output to store files
    # that we'll be linking to -- not the embedded images.
    destdir = os.path.abspath(os.path.join(setup.app.builder.outdir, outdir))
    if not os.path.exists(destdir):
        cbook.mkdirs(destdir)

    # Properly indent the caption
    caption = '\n'.join(template_content_indent + line.strip()
                        for line in caption.split('\n'))

    # Generate the figures, and return the number of them
    num_figs = render_figures(plot_path, function_name, plot_code, tmpdir,
                              destdir, formats)

    # Now start generating the lines of output
    lines = []

    if plot_code is None:
        shutil.copyfile(plot_path, os.path.join(destdir, fname))

    if options.has_key('include-source'):
        if plot_code is None:
            lines.extend(
                ['.. include:: %s' % os.path.join(setup.app.builder.srcdir, plot_path),
                 '    :literal:'])
            if options.has_key('encoding'):
                lines.append('    :encoding: %s' % options['encoding'])
                del options['encoding']
        else:
            lines.extend(['::', ''])
            lines.extend(['    %s' % row.rstrip()
                          for row in plot_code.split('\n')])
        lines.append('')
        del options['include-source']
    else:
        lines = []

    if num_figs > 0:
        options = ['%s:%s: %s' % (template_content_indent, key, val)
                   for key, val in options.items()]
        options = "\n".join(options)

        for i in range(num_figs):
            if num_figs == 1:
                outname = basename
            else:
                outname = "%s_%02d" % (basename, i)

            # Copy the linked-to files to the destination within the build tree,
            # and add a link for them
            links = []
            if plot_code is None:
                links.append('`source code <%(linkdir)s/%(basename)s.py>`__')
            for format, dpi in formats[1:]:
                links.append('`%s <%s/%s.%s>`__' % (format, linkdir, outname, format))
            if len(links):
                links = '[%s]' % (', '.join(links) % locals())
            else:
                links = ''

            lines.extend((template % locals()).split('\n'))
    else:
        lines.extend((exception_template % locals()).split('\n'))

    if len(lines):
        state_machine.insert_input(
            lines, state_machine.input_lines.source(0))

    return []

def plot_directive(name, arguments, options, content, lineno,
                   content_offset, block_text, state, state_machine):
    """
    Handle the arguments to the plot directive.  The real work happens
    in _plot_directive.
    """
    # The user may provide a filename *or* Python code content, but not both
    if len(arguments):
        plot_path = directives.uri(arguments[0])
        basedir = relpath(os.path.dirname(plot_path), setup.app.builder.srcdir)

        # If there is content, it will be passed as a caption.

        # Indent to match expansion below.  XXX - The number of spaces matches
        # that of the 'options' expansion further down.  This should be moved
        # to common code to prevent them from diverging accidentally.
        caption = '\n'.join(content)

        # If the optional function name is provided, use it
        if len(arguments) == 2:
            function_name = arguments[1]
        else:
            function_name = None

        return _plot_directive(plot_path, basedir, function_name, None, caption,
                               options, state_machine)
    else:
        plot_code = '\n'.join(content)

        # Since we don't have a filename, use a hash based on the content
        plot_path = md5(plot_code).hexdigest()[-10:]

        return _plot_directive(plot_path, 'inline', None, plot_code, '', options,
                               state_machine)

def mark_plot_labels(app, document):
    """
    To make plots referenceable, we need to move the reference from
    the "htmlonly" (or "latexonly") node to the actual figure node
    itself.
    """
    for name, explicit in document.nametypes.iteritems():
        if not explicit:
            continue
        labelid = document.nameids[name]
        if labelid is None:
            continue
        node = document.ids[labelid]
        if node.tagname in ('html_only', 'latex_only'):
            for n in node:
                if n.tagname == 'figure':
                    sectname = name
                    for c in n:
                        if c.tagname == 'caption':
                            sectname = c.astext()
                            break

                    node['ids'].remove(labelid)
                    node['names'].remove(name)
                    n['ids'].append(labelid)
                    n['names'].append(name)
                    document.settings.env.labels[name] = \
                        document.settings.env.docname, labelid, sectname
                    break

def setup(app):
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir

    options = {'alt': directives.unchanged,
               'height': directives.length_or_unitless,
               'width': directives.length_or_percentage_or_unitless,
               'scale': directives.nonnegative_int,
               'align': align,
               'class': directives.class_option,
               'include-source': directives.flag,
               'encoding': directives.encoding }

    app.add_directive('plot', plot_directive, True, (0, 2, 0), **options)
    app.add_config_value(
        'plot_formats',
        [('png', 80), ('hires.png', 200), ('pdf', 50)],
        True)

    app.connect('doctree-read', mark_plot_labels)



#!/usr/bin/python

#Imports as before:
import numpy as np
from matplotlib.pyplot import figure,legend
from matplotlib.mlab import csv2rec
from nitime.timeseries import TimeSeries
from nitime.utils import percent_change
import nitime.viz
reload(nitime.viz)
from nitime.viz import drawmatrix_channels

#This time Import the coherence analyzer 
from nitime.analysis import CoherenceAnalyzer

#This part is the same as before
TR=1.89
data_rec = csv2rec('data/fmri_timeseries.csv')
roi_names= np.array(data_rec.dtype.names)
n_samples = data_rec.shape[0]
data = np.zeros((len(roi_names),n_samples))

for n_idx, roi in enumerate(roi_names):
   data[n_idx] = data_rec[roi]

data = percent_change(data)
T = TimeSeries(data,sampling_interval=TR)
T.metadata['roi'] = roi_names 
C = CoherenceAnalyzer(T)

#We look only at frequencies between 0.02 and 0.15 (the physiologically
#relevant band, see http://imaging.mrc-cbu.cam.ac.uk/imaging/DesignEfficiency:
freq_idx = np.where((C.frequencies>0.02) * (C.frequencies<0.15))[0]

#Extract the coherence and average across these frequency bands: 
coh = np.mean(C.coherence[:,:,freq_idx],-1) #Averaging on the last dimension 
drawmatrix_channels(coh,roi_names,size=[10.,10.],color_anchor=0)




"""Code taken from the Python2.6 standard library for backwards compatibility.

This is just so we can use 2.6 features when running in 2.5, the code below is
copied verbatim from the stdlib's collections and doctest modules.
"""

from keyword import iskeyword as _iskeyword
from operator import itemgetter as _itemgetter
import sys as _sys

def namedtuple(typename, field_names, verbose=False):
    """Returns a new subclass of tuple with named fields.

    >>> Point = namedtuple('Point', 'x y')
    >>> Point.__doc__                   # docstring for the new class
    'Point(x, y)'
    >>> p = Point(11, y=22)             # instantiate with positional args or keywords
    >>> p[0] + p[1]                     # indexable like a plain tuple
    33
    >>> x, y = p                        # unpack like a regular tuple
    >>> x, y
    (11, 22)
    >>> p.x + p.y                       # fields also accessable by name
    33
    >>> d = p._asdict()                 # convert to a dictionary
    >>> d['x']
    11
    >>> Point(**d)                      # convert from a dictionary
    Point(x=11, y=22)
    >>> p._replace(x=100)               # _replace() is like str.replace() but targets named fields
    Point(x=100, y=22)

    """

    # Parse and validate the field names.  Validation serves two purposes,
    # generating informative error messages and preventing template injection attacks.
    if isinstance(field_names, basestring):
        field_names = field_names.replace(',', ' ').split() # names separated by whitespace and/or commas
    field_names = tuple(map(str, field_names))
    for name in (typename,) + field_names:
        if not all(c.isalnum() or c=='_' for c in name):
            raise ValueError('Type names and field names can only contain alphanumeric characters and underscores: %r' % name)
        if _iskeyword(name):
            raise ValueError('Type names and field names cannot be a keyword: %r' % name)
        if name[0].isdigit():
            raise ValueError('Type names and field names cannot start with a number: %r' % name)
    seen_names = set()
    for name in field_names:
        if name.startswith('_'):
            raise ValueError('Field names cannot start with an underscore: %r' % name)
        if name in seen_names:
            raise ValueError('Encountered duplicate field name: %r' % name)
        seen_names.add(name)

    # Create and fill-in the class template
    numfields = len(field_names)
    argtxt = repr(field_names).replace("'", "")[1:-1]   # tuple repr without parens or quotes
    reprtxt = ', '.join('%s=%%r' % name for name in field_names)
    dicttxt = ', '.join('%r: t[%d]' % (name, pos) for pos, name in enumerate(field_names))
    template = '''class %(typename)s(tuple):
        '%(typename)s(%(argtxt)s)' \n
        __slots__ = () \n
        _fields = %(field_names)r \n
        def __new__(_cls, %(argtxt)s):
            return _tuple.__new__(_cls, (%(argtxt)s)) \n
        @classmethod
        def _make(cls, iterable, new=tuple.__new__, len=len):
            'Make a new %(typename)s object from a sequence or iterable'
            result = new(cls, iterable)
            if len(result) != %(numfields)d:
                raise TypeError('Expected %(numfields)d arguments, got %%d' %% len(result))
            return result \n
        def __repr__(self):
            return '%(typename)s(%(reprtxt)s)' %% self \n
        def _asdict(t):
            'Return a new dict which maps field names to their values'
            return {%(dicttxt)s} \n
        def _replace(_self, **kwds):
            'Return a new %(typename)s object replacing specified fields with new values'
            result = _self._make(map(kwds.pop, %(field_names)r, _self))
            if kwds:
                raise ValueError('Got unexpected field names: %%r' %% kwds.keys())
            return result \n
        def __getnewargs__(self):
            return tuple(self) \n\n''' % locals()
    for i, name in enumerate(field_names):
        template += '        %s = _property(_itemgetter(%d))\n' % (name, i)
    if verbose:
        print template

    # Execute the template string in a temporary namespace and
    # support tracing utilities by setting a value for frame.f_globals['__name__']
    namespace = dict(_itemgetter=_itemgetter, __name__='namedtuple_%s' % typename,
                     _property=property, _tuple=tuple)
    try:
        exec template in namespace
    except SyntaxError, e:
        raise SyntaxError(e.message + ':\n' + template)
    result = namespace[typename]

    # For pickling to work, the __module__ variable needs to be set to the frame
    # where the named tuple is created.  Bypass this step in enviroments where
    # sys._getframe is not defined (Jython for example).
    if hasattr(_sys, '_getframe'):
        result.__module__ = _sys._getframe(1).f_globals.get('__name__', '__main__')

    return result


TestResults = namedtuple('TestResults', 'failed attempted')



""" Input and output for fmri data files"""

try:
    from nibabel import load
except ImportError: 
        print "nibabel required for fmri I/O"

import nitime.timeseries as ts 
import nitime.analysis as tsa
import numpy as np

def time_series_from_file(nifti_files,coords,TR,normalize=None,average=False):
    """ Make a time series from a Analyze file, provided coordinates into the
            file 

    Parameters
    ----------

    nifti_files: a string or a list of strings.

           The full path(s) to the file(s) from which the time-series is (are)
           extracted
     
    coords: ndarray
        x,y,z (inplane,inplane,slice) coordinates of the ROI from which the
        time-series is to be derived.
        
    TR: float, optional
        TR, if different from the one which can be extracted from the nifti
        file header

    normalize: Whether to normalize the activity in each voxel, defaults to
        None, in which case the original fMRI signal is used. Other options
        are: 'percent': the activity in each voxel is converted to percent
        change, relative to this scan. 'zscore': the activity is converted to a
        zscore relative to the mean and std in this voxel in this scan.

    average: bool, optional whether to average the time-series across the
           voxels in the ROI (assumed to be the first dimension). In which
           case, TS.data will be 1-d

    
    Returns
    -------

    time-series object

    Note
    ----

    Normalization occurs before averaging on a voxel-by-voxel basis, followed
    by the averaging. 

    """
    if normalize is not None:
        if normalize not in ('percent','zscore'):
            raise ValueError("Normalization of fMRI time-series can only be done using 'percent' or 'zscore' as input")
    #If just one string was provided:
    if isinstance(nifti_files,str):
        im = load(nifti_files)
        data = im.get_data()

        out_data = np.asarray(data[coords[0],coords[1],coords[2]])

        tseries = ts.TimeSeries(out_data,sampling_interval=TR)

        if normalize=='percent':
            tseries = tsa.NormalizationAnalyzer(tseries).percent_change
        elif normalize=='zscore':
            tseries = tsa.NormalizationAnalyzer(tseries).z_score

        if average:
            tseries.data = np.mean(tseries.data,0)
            
    #Otherwise loop over the files and concatenate:
    else:
        tseries_list = []
        for f in nifti_files:
            im = load(f)
            data = im.get_data()

            out_data = np.asarray(data[coords[0],coords[1],coords[2]])

            tseries_list.append(ts.TimeSeries(out_data,sampling_interval=TR))

            if normalize=='percent':
                tseries_list[-1] = tsa.NormalizationAnalyzer(tseries_list[-1]).percent_change
            elif normalize=='zscore':
                tseries_list[-1] = tsa.NormalizationAnalyzer(tseries_list[-1]).z_score

            if average:
                tseries_list[-1].data = np.mean(tseries_list[-1].data,0)

        tseries = ts.concatenate_time_series(tseries_list)
    return tseries


def nifti_from_time_series(volume,coords,time_series,nifti_path):
    """Makes a Nifti file out of a time_series object

    Parameters
    ----------

    volume: list (3-d, or 4-d)
        The total size of the nifti image to be created

    coords: 3*n_coords array
        The coords into which the time_series will be inserted. These need to
        be given in the order in which the time_series is organized

    time_series: a time-series object
       The time-series to be inserted into the file

    nifti_path: the full path to the file name which will be created
    
       """
    # XXX Implement! 
    raise NotImplementedError



"""Base classes for generic time series analysis.

The classes implemented here are meant to provide fairly basic objects for
managing time series data.  They should serve mainly as data containers, with
only minimal algorithmic functionality.

In the timeseries subpackage, there is a separate library of algorithms, and
the classes defined here mostly delegate any computational facilitites they may
have to that library.

Over time, it is OK to add increasingly functionally rich classes, but only
after their design is well proven in real-world use.

"""
#-----------------------------------------------------------------------------
# Public interface
#-----------------------------------------------------------------------------
__all__ = ['time_unit_conversion',
           'TimeSeriesInterface',
           'TimeSeries',
           'TimeInterface',
           'UniformTime',
           'TimeArray',
           'Epochs',
           'Events'
           ]
#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import warnings
import numpy as np


# Our own
from nitime import descriptors as desc

#-----------------------------------------------------------------------------
# Module globals
#-----------------------------------------------------------------------------

# These are the valid names for time units, taken from the Numpy date/time
# types specification document.  They conform to SI nomenclature where
# applicable.

# Most uses of this are membership checks, so we make a set for fast
# validation.  But we create them first as a list so we can print an ordered
# and easy to read error message.

time_unit_conversion = {
                        'ps':1, #picosecond 
                        'ns':10**3,  # nanosecond
                        'us':10**6,  # microsecond
                        'ms':10**9,  # millisecond
                        's':10**12,   # second
                        None:10**12, #The default is seconds (when constructor
                                     #doesn't get any input, it defaults to
                                     #None)
                        'm':60*10**12,   # minute
                        'h':3600*10**12,   # hour
                        'D':24*3600*10**12,   # day
                        'W':7*24*3600*10**12,   # week #This is not an SI unit
                        }

# The basic resolution: 
base_unit = 'ps'


#-----------------------------------------------------------------------------
# Class declarations
#-----------------------------------------------------------------------------

##Time: 
class TimeInterface(object):
    """ The minimal object interface for time representations

    This should be thought of as an abstract base class. """

    time_unit = None
    
class TimeArray(np.ndarray,TimeInterface):
    """Base-class for time representations, implementing the TimeInterface"""  
    def __new__(cls, data, time_unit=None, copy=True):
        """XXX Write a doc-string - in particular, mention the the default
        time-units to be used are seconds (which is why it is set to None) """ 

        # Check that the time units provided are sensible: 
        if time_unit not in time_unit_conversion:
             raise ValueError('Invalid time unit %s, must be one of %s' %
                             (time_unit,time_unit_conversion.keys()))         

        conv_fac = time_unit_conversion[time_unit]

        # We can only honor the copy flag in a very narrow set of cases
        # if data is already an TimeArray or if data is an ndarray with
        # dtype=int64
        if copy==False:
            if not getattr(data, 'dtype', None) == np.int64:
                raise ValueError('When copy flag is set to False, must provide a TimeArray object, or int64 times, in %s'%base_unit)
            time = np.array(data,copy=False)
        else:
            if isinstance(data, TimeInterface):
                time = data.copy()
            else:
                data_arr = np.asarray(data)
                if issubclass(data_arr.dtype.type,np.integer):
                    # If this is an array of integers, cast to 64 bit integer
                    # and convert to the base_unit.
                    #XXX This will fail when even 64 bit is not large enough to
                    # avoid wrap-around (When you try to make more than 10**6
                    # seconds). XXX this should be mentioned in the docstring
                    time = data_arr.astype(np.int64)*conv_fac
                else:
                    # Otherwise: first convert, round and then cast to 64 
                    time=(data_arr*conv_fac).round().astype(np.int64)

        # Make sure you have an array on your hands (for example, if you input
        # an integer, you might have reverted to an integer when multiplying
        # with the conversion factor:            
        time = np.asarray(time).view(cls)

        # Make sure time is one-dimensional or 0-d
        if time.ndim > 1:
            raise ValueError('TimeArray can only be one-dimensional or 0-d')
        
        #if time_unit is None and isinstance(data, TimeArray):
        #    time_unit = data.time_unit

        if time_unit is None:
            time_unit = 's'

        time.time_unit = time_unit
        time._conversion_factor = conv_fac
        return time
    
    def __array_wrap__(self, out_arr, context=None):
        # When doing comparisons between TimeArrays, make sure that you return a
        # boolean array, not a time array: 
        if out_arr.dtype==bool:
            return np.asarray(out_arr)
        else:
            return np.ndarray.__array_wrap__(self, out_arr, context)

    def __array_finalize__(self,obj):
        """XXX """
        # Make sure that the TimeArray has the time units set (and not equal to
        # None): 
        if not hasattr(self, 'time_unit') or self.time_unit is None:
            if hasattr(obj, 'time_unit'): # looks like view cast
                self.time_unit = obj.time_unit
            else:
                self.time_unit = 's'

        # Make sure that the conversion factor is set properly: 
        if not hasattr(self,'_conversion_factor'):
            if hasattr(obj,'_conversion_factor'):
                self._conversion_factor = obj._conversion_factor
            else:
                self._conversion_factor=time_unit_conversion[self.time_unit]

    def __repr__(self):
       """Pass it through the conversion factor"""

       # If the input is a single int/float (with no shape) return a 'scalar'
       # time-point: 
       if self.shape == ():
           return "%r %s"%(int(self)/float(self._conversion_factor),
                           self.time_unit)
       
       # Otherwise, return the TimeArray representation:
       else:
           return np.ndarray.__repr__(self/float(self._conversion_factor)
            )[:-1] + ", time_unit='%s')" % self.time_unit

    def __getitem__(self,key):
        # return scalar TimeArray in case key is integer
        if isinstance(key,int):
            return self[[key]].reshape(())
        elif isinstance(key,float):
            return self.at(key)
        elif isinstance(key,Epochs):
            return self.during(key)
        else:
            return np.ndarray.__getitem__(self,key)

    def __setitem__(self,key,val):
        
       # look at the units - convert the values to what they need to be (in the
       # base_unit) and then delegate to the ndarray.__setitem__
       if not hasattr(val,'_conversion_factor'):
           val *= self._conversion_factor
       return np.ndarray.__setitem__(self,key,val)
    
    def index_at(self,t,tol=None):
        """ Returns the integer indices that corresponds to the time t"""
        t_e = TimeArray(t,time_unit=self.time_unit)
        d = np.abs(self-t_e)
        if tol is None:
            idx=np.where(d==d.min())
        else:
            # tolerance is converted into a time-array, so that it does the
            # right thing:
            tol = TimeArray(tol,time_unit=self.time_unit)
            idx=np.where(d<=tol)            

        return idx

    def slice_during(self,e):
        """ Returns the slice that corresponds to Epoch e"""

        if not isinstance(e,Epochs):
            raise ValueError, 'e has to be of Epochs type'

        if e.data.ndim > 0:
            raise NotImplementedError, 'e has to be a scalar Epoch'

        if self.ndim != 1:
            return NotImplementedError, 'slicing only implemented for 1-d TimeArrays'
        i_start = self.index_at(e.start)[0].max()
        i_stop = self.index_at(e.stop)[0].min()
        if e.stop != self[i_stop]:
            i_stop += 1

        return slice(i_start,i_stop)

    def at(self,t,tol=None):
        """ Returns the values of the TimeArray object at time t"""
        return self[self.index_at(t,tol=tol)]

    def during(self,e):
        """ Returns the values of the TimeArray object during Epoch e"""

        if not isinstance(e,Epochs):
            raise ValueError, 'e has to be of Epochs type'

        if e.data.ndim > 0:
            ## TODO: Implement slicing with 1-d Epochs array, 
            ## resulting in (ragged/jagged) 2-d TimeArray
            raise NotImplementedError, 'e has to be a scalar Epoch'

        return self[self.slice_during(e)]

##     def min(self,axis=None,out=None):
##         """Returns the minimal time"""
##         # this is a quick fix to return a time and will
##         # be obsolete once we use proper time dtypes
##         if axis is not None:
##             raise NotImplementedError, 'axis argument not implemented'
##         if out is not None:
##             raise NotImplementedError, 'out argument not implemented'
##         if self.ndim:
##             return self[self.argmin()]
##         else:
##             return self

    def max(self,axis=None,out=None):
        """Returns the maximal time"""
        # this is a quick fix to return a time and will
        # be obsolete once we use proper time dtypes
        if axis is not None:
            raise NotImplementedError, 'axis argument not implemented'
        if out is not None:
            raise NotImplementedError, 'out argument not implemented'
        if self.ndim:
            return self[self.argmax()]
        else:
            return self

    def convert_unit(self,time_unit):
        """Convert from one time unit to another in place"""

        self.time_unit=time_unit
        self._conversion_factor = time_unit_conversion[time_unit]

        
    def __div__(self,d):
        """Division by another time object eliminates units """
        if isinstance(d,TimeInterface):
            return np.divide(np.array(self),np.array(d).astype(float))
        else: 
            return np.divide(self,d)           
        
    
class UniformTime(np.ndarray,TimeInterface):
    """ A representation of time sampled uniformly

    Parameters
    ----------
    
    length: int, the number of items in the time-array

    duration: float, the duration to be represented (given in the time-unit) of
    the array. If this item is an TimeArray, the units of the UniformTime
    array resulting will 'inherit' the units of the duration. Otherwise, the
    unit of the UniformTime will be set by that kwarg

    sampling_rate: float, the sampling rate (in 1/time-unit)

    sampling_interval: float, the inverse of the sampling_interval     

    t0: the value of the first time-point in the array (in time-unit)

    time_unit:

    copy: whether to make a copy of not. Needs to be set to False 

    
    

    XXX continue writing this
    """

    def __new__(cls,data=None,length=None,duration=None,sampling_rate=None,
                sampling_interval=None,t0=0,time_unit=None, copy=False):
        """Create a new UniformTime """

        # Sanity checks. There are different valid combinations of inputs
        tspec = tuple(x is not None for x in
                      [sampling_interval,sampling_rate,length,duration])

        # Used in converting tspecs to human readable form
        tspec_arg_names = ['sampling_interval','sampling_rate','length','duration']

        # The valid configurations 
        valid_tspecs=[
            # interval, length:
            (True,False,True,False),
            # interval, duration:
            (True,False,False,True),
            # rate, length:
            (False,True,True,False),
            # rate, duration:
            (False,True,False,True),
            # length, duration:
            (False,False,True,True)
            ]
        
        if isinstance(data,UniformTime):
            # Assuming data was given, some other tspecs become valid:
            tspecs_w_data=dict(
                    nothing=(False,False,False,False),
                    sampling_interval= (True,False,False,False),
                    sampling_rate= (False,True,False,False),
                    length= (False,False,True,False),
                    duration=(False,False,False,True))
            # preserve the order of the keys
            valid_tspecs.append( tspecs_w_data['nothing'])
            for name in tspec_arg_names:
                valid_tspecs.append( tspecs_w_data[name])

        if (tspec not in valid_tspecs):
            l = ['sampling_interval','sampling_rate','length','duration']
            #args = [arg for t,arg in zip(tspec,l) if t]
            raise ValueError("Invalid time specification.\n" +
                "You provided: %s \n"
                "%s \nsee docstring for more info." 
                %(str_tspec(tspec,tspec_arg_names), str_valid_tspecs(valid_tspecs,tspec_arg_names)))
            
        if isinstance(data,UniformTime):
            # Get attributes from the UniformTime object and transfer those over:
            if tspec==tspecs_w_data['nothing']:
                sampling_rate=data.sampling_rate
                duration = data.duration
            elif tspec==tspecs_w_data['sampling_interval']:
                duration==data.duration
            elif tspec==tspecs_w_data['sampling_rate']:
                if isinstance(sampling_rate,Frequency):
                    sampling_interval=sampling_rate.to_period()
                else:
                    sampling_interval = 1.0/sampling_rate
                duration=data.duration
            elif tspec==tspecs_w_data['length']:
                duration=length*data.sampling_interval
                sampling_rate=data.sampling_rate
            elif tspec==tspecs_w_data['duration']:
                sampling_rate=data.sampling_rate
            if time_unit is None:
                # If the user didn't ask to change the time-unit, use the
                # time-unit from the object you got:
                time_unit = data.time_unit
        
        # Check that the time units provided are sensible: 
        if time_unit not in time_unit_conversion:
            raise ValueError('Invalid time unit %s, must be one of %s' %
                         (time_unit,time_unit_conversion.keys()))         

        # Make sure you have a time unit:
        if time_unit is None:
            #If you gave us a duration with time_unit attached 
            if isinstance(duration,TimeInterface):
                time_unit = duration.time_unit
            #Otherwise, you might have given us a sampling_interval with a
            #time_unit attached:
            elif isinstance(sampling_interval,TimeInterface):
                time_unit = sampling_interval.time_unit
            else:
                time_unit = 's'

        # Calculate the sampling_interval or sampling_rate:
        if sampling_interval is None:
            if isinstance(sampling_rate,Frequency):
                c_f = time_unit_conversion[time_unit]
                sampling_interval=sampling_rate.to_period()/float(c_f)
            elif sampling_rate is None:
                sampling_interval = float(duration)/length
                sampling_rate = Frequency(1.0/sampling_interval,
                                             time_unit=time_unit)
            else:
                c_f = time_unit_conversion[time_unit]
                sampling_rate = Frequency(sampling_rate,time_unit='s')
                sampling_interval = sampling_rate.to_period()/float(c_f)
        else:
            if isinstance(sampling_interval,TimeInterface):
                c_f = time_unit_conversion[sampling_interval.time_unit]
                sampling_rate = Frequency(1.0/(float(sampling_interval)/c_f),
                                          time_unit=sampling_interval.time_unit)
            else:
                sampling_rate = Frequency(1.0/sampling_interval,
                                          time_unit=time_unit)

        # Calculate the duration, if that is not defined:
        if duration is None:
            duration=length*sampling_interval


        # 'cast' the time inputs as TimeArray
        duration=TimeArray(duration,time_unit=time_unit)
        #XXX If data is given - the t0 should be taken from there:
        t0=TimeArray(t0,time_unit=time_unit)
        sampling_interval=TimeArray(sampling_interval,time_unit=time_unit)

        # Check that the inputs are consistent, before making the array
        # itself:
        if duration<sampling_interval:
            raise ValueError('length/duration too short for the sampling_interval/sampling_rate')
        
        # in order for time[-1]-time[0]==duration to be true (which it should)
        # add the samling_interval to the stop value: 
        # time = np.arange(np.int64(t0),np.int64(t0+duration+sampling_interval),
        #                  np.int64(sampling_interval),dtype=np.int64)

        # But it's unclear whether that's really the behavior we want?
        time = np.arange(np.int64(t0),np.int64(t0+duration),
                         np.int64(sampling_interval),dtype=np.int64)

        time = np.asarray(time).view(cls)
        time.time_unit=time_unit
        time._conversion_factor=time_unit_conversion[time_unit]
        time.duration = duration
        time.sampling_rate=Frequency(sampling_rate)
        time.sampling_interval=sampling_interval
        time.t0 = t0
        
        return time

    def __array_wrap__(self, out_arr, context=None):
        # When doing comparisons between UniformTime, make sure that you retun a
        # boolean array, not a time array: 
        if out_arr.dtype==bool:
            return np.asarray(out_arr)
        else:
            return np.ndarray.__array_wrap__(self, out_arr, context)

    def __array_finalize__(self,obj):
        """XXX """
        # Make sure that the UniformTime has the time units set (and not equal to
        # None): 
        if not hasattr(self, 'time_unit') or self.time_unit is None:
            if hasattr(obj, 'time_unit'): # looks like view cast
                self.time_unit = obj.time_unit
            else:
                self.time_unit = 's'

        # Make sure that the conversion factor is set properly: 
        if not hasattr(self,'_conversion_factor'):
            if hasattr(obj,'_conversion_factor'):
                self._conversion_factor = obj._conversion_factor
            else:
                self._conversion_factor=time_unit_conversion[self.time_unit]

    def __repr__(self):
       """Pass it through the conversion factor"""

       #If the input is a single int/float (with no shape) return a 'scalar'
       #time-point: 
       if self.shape == ():
           return "%r %s"%(int(self)/float(self._conversion_factor),
                           self.time_unit)
       
       #Otherwise, return the UniformTime representation:
       else:
           return np.ndarray.__repr__(self/float(self._conversion_factor)
            )[:-1] + ", time_unit='%s')" % self.time_unit

    def __getitem__(self,key):
        # return scalar TimeArray in case key is integer
        if isinstance(key,int):
            return self[[key]].reshape(()).view(TimeArray)
        elif isinstance(key,float) or isinstance(key, TimeInterface):
            return self.at(key)
        elif isinstance(key,Epochs):
            return self.during(key)
        else:
            return np.ndarray.__getitem__(self,key)

    def __setitem__(self,key,val):
       # look at the units - convert the values to what they need to be (in the
       # base_unit) and then delegate to the ndarray.__setitem__    
       if not hasattr(val,'_conversion_factor'):
           val *= self._conversion_factor
       return np.ndarray.__setitem__(self,key,val)

    def index_at(self,t,boolean=False):
        """Find the index that corresponds to the time bin containing t

           Returns boolean mask if boolean=True and integer indeces otherwise.
        """

        # cast t into time
        ta = TimeArray(t,time_unit=self.time_unit)

        # check that index is within range
        if ta.min() < self.t0 or ta.max() >= self.t0 + self.duration:
            raise ValueError, 'index out of range'
        
        idx = ta.view(np.ndarray)//int(self.sampling_interval) 
        idx -= self.t0/self._conversion_factor
        if boolean:
            bool_idx = np.zeros(len(self),dtype=bool)
            bool_idx[idx] = True
            return bool_idx
        elif ta.ndim == 0:
            return idx[()]
        else:
            return idx

    def slice_during(self,e):
        """ Returns the slice that corresponds to Epoch e"""

        if not isinstance(e,Epochs):
            raise ValueError, 'e has to be of Epochs type'

        if e.data.ndim > 0:
            raise NotImplementedError, 'e has to be a scalar Epoch'

        if self.ndim != 1:
            return NotImplementedError, 'slicing only implemented for 1-d TimeArrays'
        i_start = self.index_at(e.start)
        i_stop = self.index_at(e.stop)
        if e.stop != self[i_stop]:
            i_stop += 1

        return slice(i_start,i_stop)

    def at(self,t):
        """ Returns the values of the UniformTime object at time t"""
        return TimeArray(self[self.index_at(t)],time_unit=self.time_unit)

    def during(self,e):
        """ Returns the values of the UniformTime object during Epoch e"""

        if not isinstance(e,Epochs):
            raise ValueError, 'e has to be of Epochs type'

        if e.data.ndim > 0:
            raise NotImplementedError, 'e has to be a scalar Epoch'

        return self[self.slice_during(e)]

    def min(self,axis=None,out=None):
        """Returns the minimal time"""
        # this is a quick fix to return a time and will
        # be obsolete once we use proper time dtypes
        if axis is not None:
            raise NotImplementedError, 'axis argument not implemented'
        if out is not None:
            raise NotImplementedError, 'out argument not implemented'
        if self.ndim:
            return self[self.argmin()]
        else:
            return self

    def max(self,axis=None,out=None):
        """Returns the maximal time"""
        # this is a quick fix to return a time and will
        # be obsolete once we use proper time dtypes
        if axis is not None:
            raise NotImplementedError, 'axis argument not implemented'
        if out is not None:
            raise NotImplementedError, 'out argument not implemented'
        if self.ndim:
            return self[self.argmax()]
        else:
            return self

    def __div__(self,d):
        """Division by another time object eliminates units """
        if isinstance(d,TimeInterface):
            return np.divide(np.array(self),np.array(d).astype(float))
        else: 
            return np.divide(self,d)           


##Frequency:

class Frequency(float):
    """A class for representation of the frequency (in Hz) """

    def __new__(cls,f,time_unit='s'):
        """Initialize a frequency object """

        tuc = time_unit_conversion
        scale_factor = (float(tuc['s'])/tuc[time_unit])
        #If the input is a Frequency object, it is already in Hz: 
        if isinstance(f,Frequency)==False:
            #But otherwise convert to Hz:
            f = f*scale_factor

        freq = super(Frequency,cls).__new__(cls,f)
        freq._time_unit = time_unit

        return freq
    
    def __repr__(self):
        
        return str(self) + ' Hz'

    def to_period(self,time_unit=base_unit):
        """Convert the value of a frequency to the corresponding period
        (defaulting to a representation in the base_unit)

        """
        tuc = time_unit_conversion
        scale_factor = (float(tuc['s'])/tuc[time_unit])
        
        return np.int64((1/self)*scale_factor)
        
##Time-series: 
class TimeSeriesInterface(TimeInterface):
    """The minimally agreed upon interface for all time series.

    This should be thought of as an abstract base class.
    """
    time = None
    data = None
    metadata = None
    

class TimeSeriesBase(object):
    """Base class for time series, implementing the TimeSeriesInterface."""

    def __init__(self, data, time_unit, metadata=None):
        """Common constructor shared by all TimeSeries classes."""
        # Check that sensible time units were given
        if time_unit not in time_unit_conversion:
            raise ValueError('Invalid time unit %s, must be one of %s' %
                             (time_unit,time_unit_conversion.keys()))
        
        #: the data is an arbitrary numpy array
        self.data = np.asarray(data)
        self.time_unit = time_unit

        # Every instance carries an empty metadata dict, which we promise never
        # to touch.  This reserves this name as a user area for extra
        # information without the danger of name clashes in the future.
        if metadata is None:
            self.metadata = {}
        else:
            self.metadata = metadata

    def __len__(self):
        """Return the length of the time series."""
        return self.data.shape[-1]


    def _validate_dimensionality(self):
        """Check that the data and time have the proper dimensions.
        """

        if self.time.ndim != 1:
            raise ValueError("time array must be one-dimensional")
        npoints = self.data.shape[-1]
        if npoints != len(self.time):
            raise ValueError("mismatch of time and data dimensions")


    def __getitem__(self,key):
        """use fancy time-indexing (at() method).""" 
        if isinstance(key,TimeInterface):
            return self.at(key)
        elif isinstance(key,Epochs):
            return self.during(key)
        elif self.data.ndim==1:
            return self.data[key] #time is the last dimension
        else:
            return self.data[:,key] #time is the last dimension

    def __repr__(self):
        rep = self.__class__.__name__ + ":" 
        return rep + self.time.__repr__() + self.data.T.__repr__()

        
class TimeSeries(TimeSeriesBase):
    """Represent data collected at uniform intervals.
    """

    @desc.setattr_on_read
    def time(self):
        """Construct time array for the time-series object. This holds a
    UniformTime object, with properties derived from the TimeSeries
    object"""
        return UniformTime(length=self.__len__(),t0=self.t0,
                           sampling_interval=self.sampling_interval,
                           time_unit=self.time_unit)

    #XXX This should call the constructor in an appropriate way, when provided
    #with a UniformTime object and data, so that you don't need to deal with
    #the constructor itself:  
    @staticmethod
    def from_time_and_data(time, data):
        return TimeSeries.__init__(data, time=time)
        
    
    
    def __init__(self, data, t0=None, sampling_interval=None,
                 sampling_rate=None, duration=None, time=None, time_unit='s',
                 metadata=None):
        """Create a new TimeSeries.

        This class assumes that data is uniformly sampled, but you can specify
        the sampling in one of three (mutually exclusive) ways:

        - sampling_interval [, t0]: data sampled starting at t0, equal
          intervals of sampling_interval.

        - sampling_rate [, t0]: data sampled starting at t0, equal intervals of
          width 1/sampling_rate.

        - time: a UniformTime object, in which case the TimeSeries can
          'inherit' the properties of this object.  
        
        Parameters
        ----------
        data : array_like
          Data array, interpreted as having its last dimension being time.
        sampling_interval : float
          Interval between successive time points.
        sampling_rate : float
          Inverse of the interval between successive time points.
        t0 : float
          If you provide a sampling rate, you can optionally also provide a
          starting time.
        time
          Instead of sampling rate, you can explicitly provide an object of
        class UniformTime. Note that you can still also provide a different
        sampling_rate/sampling_interval/duration to take the place of the one
        in this object, but only as long as the changes are consistent with the
        length of the data. 
        
        time_unit :  string
          The unit of time.

        Examples 
        --------

        The minimal specification of data and sampling interval:

        >>> ts = TimeSeries([1,2,3],sampling_interval=0.25)
        >>> ts.time
        UniformTime([ 0.  ,  0.25,  0.5 ], time_unit='s')
        >>> ts.t0
        0.0 s
        >>> ts.sampling_rate
        4.0 Hz

        Or data and sampling rate:
        >>> ts = TimeSeries([1,2,3],sampling_rate=2)
        >>> ts.time
        UniformTime([ 0. ,  0.5,  1. ], time_unit='s')
        >>> ts.t0
        0.0 s
        >>> ts.sampling_interval
        0.5 s

        A time series where we specify the start time and sampling interval:
        >>> ts = TimeSeries([1,2,3],t0=4.25,sampling_interval=0.5)
        >>> ts.data
        array([1, 2, 3])
        >>> ts.time
        UniformTime([ 4.25,  4.75,  5.25], time_unit='s')
        >>> ts.t0
        4.25 s
        >>> ts.sampling_interval
        0.5 s
        >>> ts.sampling_rate
        2.0 Hz

        >>> ts = TimeSeries([1,2,3],t0=4.25,sampling_rate=2.0)
        >>> ts.data
        array([1, 2, 3])
        >>> ts.time
        UniformTime([ 4.25,  4.75,  5.25], time_unit='s')
        >>> ts.t0
        4.25 s
        >>> ts.sampl
        ts.sampling_interval  ts.sampling_rate      
        >>> ts.sampling_interval
        0.5 s
        >>> ts.sampling_rate
        2.0 Hz

        """

        #If a UniformTime object was provided as input: 
        if isinstance(time,UniformTime): 
            c_fac = time._conversion_factor
            #If the user did not provide an alternative t0, get that from the
            #input: 
            if t0 is None:
                t0=time.t0
            #If the user did not provide an alternative sampling interval/rate:
            if sampling_interval is None and sampling_rate is None:
                sampling_interval = time.sampling_interval
                sampling_rate = time.sampling_rate
            #The duration can be read either from the length of the data, or
            #from the duration specified by the time-series: 
            if duration is None:
                duration=time.duration
                length = time.shape[-1]
                #If changing the duration requires a change to the
                #sampling_rate, make sure that this was explicitely required by
                #the user - if the user did not explicitely set the
                #sampling_rate, or it is inconsistent, throw an error: 
                data_len = np.array(data).shape[-1]

                if (length != data_len and
                    sampling_rate != float(data_len*c_fac)/time.duration):
                    e_s = "Length of the data (%s) " %str(len(data))  
                    e_s += "specified sampling_rate (%s) " %str(sampling_rate)
                    e_s +="do not match."
                    raise ValueError(e_s)
            #If user does not provide a     
            if time_unit is None:
                time_unit = time.time_unit

        else:    
            ##If the input was not a UniformTime, we need to check that there
            ##is enough information in the input to generate the UniformTime
            ##array.

            #There are different valid combinations of inputs
            tspec = tuple(x is not None for x in
                      [sampling_interval,sampling_rate,duration])

            tspec_arg_names = ["sampling_interval", "sampling_rate", "duration"] 

            #The valid configurations 
            valid_tspecs=[
                      #interval,length:
                      (True,False,False),
                      #interval,duration:
                      (True,False,True),
                      #rate,length:
                      (False,True,False),
                      #rate, duration:
                      (False,True,True),
                      #length,duration:
                      (False,False,True)
                      ]

            if tspec not in valid_tspecs:
                raise ValueError("Invalid time specification. \n"
                        "You provided: %s\n %s see docstring for more info." % (
                            str_tspec(tspec, tspec_arg_names),
                            str_valid_tspecs(valid_tspecs,tspec_arg_names)))


        # Make sure to grab the time unit from the inputs, if it is provided: 
        if time_unit is None:
            #If you gave us a duration with time_unit attached 
            if isinstance(duration,TimeInterface):
                time_unit = duration.time_unit
            #Otherwise, you might have given us a sampling_interval with a
            #time_unit attached:
            elif isinstance(sampling_interval,TimeInterface):
                time_unit = sampling_interval.time_unit

        
        #Calculate the sampling_interval or sampling_rate from each other and
        #assign t0, if it is not already assigned:
        if sampling_interval is None:
            if isinstance(sampling_rate,Frequency):
                c_f = time_unit_conversion[time_unit]
                sampling_interval=sampling_rate.to_period()/float(c_f)
            elif sampling_rate is None:
                data_len = np.asarray(data).shape[-1]
                sampling_interval = float(duration)/data_len
                sampling_rate = Frequency(1.0/sampling_interval,
                                             time_unit=time_unit)
            else:
                c_f = time_unit_conversion[time_unit]
                sampling_rate = Frequency(sampling_rate,time_unit='s')
                sampling_interval = sampling_rate.to_period()/float(c_f)
        else:
            if sampling_rate is None: #Only if you didn't already 'inherit'
                                      #this property from another time object
                                      #above:
                if isinstance(sampling_interval,TimeInterface):
                   c_f = time_unit_conversion[sampling_interval.time_unit]
                   sampling_rate = Frequency(1.0/(float(sampling_interval)/c_f),
                                          time_unit=sampling_interval.time_unit)
                else:
                   sampling_rate = Frequency(1.0/sampling_interval,
                                          time_unit=time_unit)
            
        #Calculate the duration, if that is not defined:
        if duration is None:
            duration=np.asarray(data).shape[-1]*sampling_interval

        if t0 is None:
           t0=0

           
        #Otherwise, you can still call the common constructor to get the real
        #object initialized, with time_unit set to None and that will generate
        #the object with time_unit set to 's':
        TimeSeriesBase.__init__(self, data, time_unit, metadata=metadata)
    
        self.time_unit = time_unit
        self.sampling_interval = TimeArray(sampling_interval,
                                           time_unit=self.time_unit) 
        self.t0 = TimeArray(t0,time_unit=self.time_unit)
        self.sampling_rate = sampling_rate
        self.duration = TimeArray(duration,time_unit=self.time_unit)

    def at(self,t,tol=None):
        """ Returns the values of the TimeArray object at time t"""
        return self.data[...,self.time.index_at(t)]

            
        #Calculate the duration, if that is not defined:
        if duration is None:
            duration=np.asarray(data).shape[-1]*sampling_interval

        if t0 is None:
           t0=0
           
        # Make sure to grab the time unit from the inputs, if it is provided: 
        if time_unit is None:
            #If you gave us a duration with time_unit attached 
            if isinstance(duration,TimeInterface):
                time_unit = duration.time_unit
            #Otherwise, you might have given us a sampling_interval with a
            #time_unit attached:
            elif isinstance(sampling_interval,TimeInterface):
                time_unit = sampling_interval.time_unit

        #Otherwise, you can still call the common constructor to get the real
        #object initialized, with time_unit set to None and that will generate
        #the object with time_unit set to 's':  
        TimeSeriesBase.__init__(self,data,time_unit)
    
        self.time_unit = time_unit
        self.sampling_interval = TimeArray(sampling_interval,
                                           time_unit=self.time_unit) 
        self.t0 = TimeArray(t0,time_unit=self.time_unit)
        self.sampling_rate = sampling_rate
        self.duration = TimeArray(duration,time_unit=self.time_unit)

    def during(self,e):
        """ Returns the TimeSeries slice corresponding to epoch e """

        if not isinstance(e,Epochs):
            raise ValueError, 'e has to be of Epochs type'

        if e.data.ndim == 0:
            return TimeSeries(data=self.data[...,self.time.slice_during(e)],
                              time_unit=self.time_unit, t0=e.offset,
                              sampling_rate=self.sampling_rate)
        else:
            # TODO: make this a more efficient implementation, naive first pass
            if (e.duration != e.duration[0]).any():
                raise ValueError("All epochs must have the same duration")
            data = np.array([self.data[...,self.time.slice_during(ep)] for ep in e])
            return TimeSeries(data=data,
                              time_unit=self.time_unit, t0=e.offset,
                              sampling_rate=self.sampling_rate)

    @property
    def shape(self):
        return self.data.shape

_epochtype = np.dtype({'names':['start','stop'],'formats':[np.int64]*2})
class Epochs():
    """Represents a time interval"""
    
    def __init__(self, t0=None, stop=None,offset=None, start=None, duration=None,
                 time_unit=None, static=None, **kwargs):
        
        # Short-circuit path for a fast initialization. This relies on `static`
        # to be a dict that contains everything that defines an Epochs class
        # XXX: add this sort of fast __init__ to all other classes
        if static is not None:
            self.__dict__.update(static)
            return

        if t0 is None and start is None:
            raise ValueError('Either start or t0 need to be specified') 
        # Normal, error checking and type converting initialization logic

        if stop is None and duration is None:
            raise ValueError('Either stop or duration have to be specified')

        if stop is not None and duration is not None:
            ### TODO: check if stop and duration are consistent
            raise ValueError('Only either stop or duration have to be specified')

        if offset is None:
            offset = 0

        t_offset = TimeArray(offset,time_unit=time_unit)

        if t_offset.ndim > 0:
            raise ValueError, 'Only scalar offset allowed'

        if t0 is None:
            t_0 = 0
        else:
            t_0 = TimeArray(t0,time_unit=time_unit)

        if start is None:
            t_start = t_0-t_offset
        else:
            t_start = TimeArray(start,time_unit=time_unit)
            
        # inherit time_unit of t_start
        self.time_unit = t_start.time_unit

        if stop is None:
            t_duration = TimeArray(duration,time_unit=time_unit)
            t_stop = t_start + t_duration
        else:
            t_stop = TimeArray(stop,time_unit=time_unit)

        if t_start.shape != t_stop.shape:
            raise ValueError, 'start and stop have to have same shape'

        if t_start.ndim == 0:
            # return a 'scalar' epoch
            self.data = np.empty(1,dtype=_epochtype).reshape(())
        elif t_start.ndim == 1:
            # return a 1-d epoch array
            self.data = np.empty(t_start.shape[0],dtype=_epochtype)
        else:
            raise ValueError, 'Only 0-dim and 1-dim start and stop times allowed'

        self.data['start'] = t_start
        self.data['stop'] = t_stop
        

        self.offset = t_offset

    # TODO: define setters for start, stop, offset attributes
    @property
    def start(self):
        return TimeArray(self.data['start'],time_unit=self.time_unit,copy=False)

    @property
    def stop(self):
        return TimeArray(self.data['stop'],time_unit=self.time_unit,copy=False)

    @desc.setattr_on_read
    def duration(self):
        """Duration array for the epoch"""
        return self.stop-self.start

    def __getitem__(self,key):
        # create the static dict needed for fast version of __init__
        static = self.__dict__.copy()
        static['data'] = self.data[key]
        return Epochs(start=None,static=static) # `start` is a required argument

    def __repr__(self):
        if self.data.ndim == 0:
            z = (self.start, self.stop)
        else:
            z = zip(self.start, self.stop)
        rep =self.__class__.__name__ + "(" + z.__repr__()
        return rep + ", as (start,stop) tuples)"

    def __len__(self):
        return len(self.data)


def str_tspec(tspec, arg_names):
    """ Turn a single tspec into human readable form"""
    # an all "False" will convert to an empty string unless we do the following
    # where we create an all False tuple of the appropriate length
    if tspec==tuple([False]*len(arg_names)):
        return "(nothing)"
    return ", ".join([arg for t,arg in zip(tspec,arg_names) if t])

def str_valid_tspecs(valid_tspecs, arg_names):
    """Given a set of valid_tspecs, return a string that turns them into
    human-readable form"""
    vargs = []
    for tsp in valid_tspecs: 
        vargs.append(str_tspec(tsp, arg_names))
    return "\n Valid time specifications are:\n\t%s" %("\n\t".join(vargs))
    
def concatenate_time_series(time_series_seq):
    """Concatenates a sequence of time-series objects in time.

    The input can be any iterable of time-series objects; metadata, sampling
    rates and other attributes are kept from the last one in the sequence.

    This one requires that all the time-series in the list have the same
    sampling rate and that all the data have the same number of items in all
    dimensions, except the time dimension"""

    # Extract the data pointer for each and build a common data block
    data = []
    metadata = {}
    for ts in time_series_seq:
        data.append(ts.data)
        metadata.update(ts.metadata)

    # Sampling interval is read from the last one
    tseries = TimeSeries(np.hstack(data),
                                sampling_interval=ts.sampling_interval,
                                metadata=metadata)
    return tseries

class Events(TimeInterface):
    """Represents timestamps and associated data """ 

    def __init__(self, time, labels=None, indices=None, time_unit=None, **data):
        # First initilaize the TimeArray from the time-stamps
        self.time = TimeArray(time,time_unit=time_unit)
        self.time_unit = self.time.time_unit

        # Make sure time is one-dimensional
        if self.time.ndim != 1:
            raise ValueError('The TimeArray provided can only be one-dimensional ')

        # Make sure all data has same length
        for check_v in data.values():
            if len(check_v) != len(self.time):
                raise ValueError('All data in the Events must be of the same length as the associated time')

        # Make sure indices have same length and are integers
        if labels is not None:
            if len(labels) != len(indices):
                raise ValueError('Labels and indices must have the same length')
            dt = [(l,np.int64) for l in labels]
        else:
            dt = np.int64
            dt = [('i%d'%i,np.int64) for i in range(len(indices or ()))] or np.int64
        self.index = np.array(zip(*(indices or ())), dtype=dt).view(np.recarray)

        #Should data be a recarray? 
##         dt = [(st,np.array(data[st]).dtype) for st in data] or None
##         self.data = np.array(zip(*data.values()), dtype=dt).view(np.recarray)
        #Or a dict? 
        self.data = dict(data)




#!/usr/bin/python

#Imports as before:
import numpy as np
from matplotlib.pyplot import figure,legend
from matplotlib.mlab import csv2rec
from nitime.timeseries import TimeSeries
from nitime.utils import percent_change
import nitime.viz
reload(nitime.viz)
from nitime.viz import drawgraph_channels,drawmatrix_channels

#This time Import the coherence analyzer 
from nitime.analysis import CoherenceAnalyzer

#This part is the same as before
TR=1.89
data_rec = csv2rec('data/fmri_timeseries.csv')
roi_names= np.array(data_rec.dtype.names)
n_samples = data_rec.shape[0]
data = np.zeros((len(roi_names),n_samples))

for n_idx, roi in enumerate(roi_names):
   data[n_idx] = data_rec[roi]

data = percent_change(data)
T = TimeSeries(data,sampling_interval=TR)
T.metadata['roi'] = roi_names 
C = CoherenceAnalyzer(T)
freq_idx = np.where((C.frequencies>0.02) * (C.frequencies<0.15))[0]

idx_lcau = np.where(roi_names=='lcau')[0]
idx_rcau = np.where(roi_names=='rcau')[0]
idx_lput = np.where(roi_names=='lput')[0]
idx_rput = np.where(roi_names=='rput')[0]

idx = np.hstack([idx_lcau,idx_rcau,idx_lput,idx_rput])
idx1 = np.vstack([[idx[i]]*4 for i in range(4)]).ravel()
idx2 = np.hstack(4*[idx])

#Extract the coherence and average across these frequency bands: 
coh = C.coherence[idx1,idx2].reshape(4,4,C.frequencies.shape[0])
coh = np.mean(coh[:,:,freq_idx],2) #Averaging on the last dimension

drawgraph_channels(coh,roi_names[idx])



#!/usr/bin/python

#Import from other libraries:
import numpy as np
from matplotlib.pyplot import figure
from matplotlib.mlab import csv2rec

#Import the time-series objects: 
from nitime.timeseries import TimeSeries 
#Import the correlation analyis object
from nitime.analysis import CorrelationAnalyzer
#Import utility functions:
from nitime.utils import percent_change
import nitime.viz
reload(nitime.viz)
from nitime.viz import drawmatrix_channels,drawgraph_channels

#This information (the sampling rate) has to be known in advance:
TR=1.89

#Load the data from the csv file:
data_rec = csv2rec('data/fmri_timeseries.csv')

#Extract information:
roi_names= np.array(data_rec.dtype.names)
n_samples = data_rec.shape[0]


#Make an empty container for the data
data = np.zeros((len(roi_names),n_samples))

for n_idx, roi in enumerate(roi_names):
   data[n_idx] = data_rec[roi]

#Normalize the data:
data = percent_change(data)

#Initialize the time-series from the normalized data:
T = TimeSeries(data,sampling_interval=TR)
T.metadata['roi'] = roi_names 

#Initialize the correlation analyzer
C = CorrelationAnalyzer(T)

#Display the correlation matrix
drawmatrix_channels(C(),roi_names,size=[10.,10.],color_anchor=0)




#Imports:
import numpy as np
#Import nitime.timeseries for representation objects:
import nitime.timeseries as ts
#Import nitime.analysis for analysis object:
import nitime.analysis as tsa
reload(tsa)
#The viz library is used for visualization:
import nitime.viz as viz

#Load the stimulus from the data file:
maxdB = 76.4286 #Taken from the spike file header
stim = np.loadtxt('data/grasshopper_stimulus1.txt')
stim = (20*1/np.log(10))*(np.log(stim[:,1]/2.0e-5))
stim = maxdB-stim.max()+stim

#Create the stimulus time-series (sampled at 20 kHz):
stim_time_series = ts.TimeSeries(t0=0,data=stim,sampling_interval=50,
                                 time_unit='us')
#Convert time-unit:
stim_time_series.time_unit='ms'
#Load the spike-times from the data file:
spike_times = np.loadtxt('data/grasshopper_spike_times1.txt')
#Initialize the Event object holding the spike-times:
spike_ev = ts.Events(spike_times,time_unit='us')

#Initialize the event-related analyzer
event_related = tsa.EventRelatedAnalyzer(stim_time_series,spike_ev,len_et=200,
                                                                  offset=-200)

#The actual STA gets calculated in this line (the call to 'event_related.eta')
#and the result gets input directly into the plotting function:
fig = viz.plot_tseries(event_related.eta,ylabel='Amplitude (dB SPL)')

#We also plot the average of the stimulus amplitude and the time of the spike,
#using dashed lines:
ax = fig.get_axes()[0]
xlim = ax.get_xlim()
ylim = ax.get_ylim()
mean_stim = np.mean(stim_time_series.data)
ax.plot([xlim[0],xlim[1]],[mean_stim,mean_stim],'k--')





